\ start by loading the constants
load ../../dataDictionary/common/common.4th

: endGroup { results | -- } results .table ;

: runQuery { walker |  -- } walker 0 0 ['] endGroup IntervalWalkQueryHT ;

\ =================== address map query ===========================

: initTspecAM { -- tspec }
  	resetColumnIndex
	TableSpecNew
		dup ft_MAC_src     addKey
		dup ft_IP_src      addKey
		dup ft_IP6_src     addKey
		dup ft_IPX_src     addKey
		dup ft_DDP_src     addKey
		dup ft_DEC4_src    addKey
		dup ft_FBCN_src    addKey
		dup ft_FLAGS	   addKey
		dup -1 TableSpecSetN
;

: latestActiveFlows s" activeWriteFlows_1.latestTime" ReadMIBVar asSeconds ;

: initTwalkAM { tspec | -- twalk }
	tspec
	db_ACTIVEFLOWS db_UNDEFINED db_UNDEFINED db_UNDEFINED
	IntervalWalkNew

	dup 0 IntervalWalkNewGroup
	dup
	latestActiveFlows
	1 minute * \ duration
	IntervalWalkAdd
;

: runAM  initTspecAM initTwalkAM runQuery ;

\ =================== mac locations query ===========================

: initTspecAL { -- tspec }
        resetColumnIndex
	TableSpecNew
		dup ft_agent_IP	   addKey
		dup ft_IF_in	   addKey
		dup ft_IF_out	   addKey
		dup ft_MAC_src     addKey
		dup ft_dsFlags64   agg_OR sort_NONE addColumn
		dup ft_ds_class    addKey
		dup ft_ds_index    addKey
		dup ft_CDP_mgmt    addKey
		dup ft_VLAN_in     addKey
		dup ft_VLAN_out    addKey
		dup ft_inferred    agg_OR sort_NONE addColumn
		dup -1 TableSpecSetN
;

: initTwalkAL { tspec | -- twalk }
	tspec
	db_ACTIVEFLOWS db_UNDEFINED db_UNDEFINED db_UNDEFINED
	IntervalWalkNew

	dup 0 IntervalWalkNewGroup
	dup
	latestActiveFlows
	1 minute *                     \ 1 minute duration
	IntervalWalkAdd
;

: runAL  initTspecAL initTwalkAL runQuery ;


\ =================== spanning tree discovery protocol query ===========================

: initTspecSTP { -- tspec }
        resetColumnIndex
	TableSpecNew
		dup ft_MAC_src     	   addKey
		dup ft_agent_IP	   	   addKey
		dup ft_ds_class	   	   addKey
		dup ft_ds_index	   	   addKey
		dup ft_IF_in	   	   addKey
		dup ft_IF_out	   	   addKey
		dup ft_STP_flags	   addKey
		dup ft_STP_root_priority   addKey
		dup ft_STP_root		   addKey
		dup ft_STP_bridge_priority addKey
		dup ft_STP_bridge	   addKey
		dup ft_STP_port		   addKey
	 	dup ft_STP_cost		   addKey
		dup ft_VLAN_in		   addKey
		dup ft_inferred            agg_OR sort_NONE addColumn
		dup -1 TableSpecSetN
;

: initTwalkSTP { tspec | -- twalk }
	tspec
	db_ACTIVEFLOWS db_UNDEFINED db_UNDEFINED db_UNDEFINED
	IntervalWalkNew

	dup 0 IntervalWalkNewGroup
	dup
	latestActiveFlows
	1 minute *                    \ 1-minute duration
	IntervalWalkAdd
;

: filterSTP
	\ filter to select only Spanning Tree multicasts:  ieee802sap = 66 & macdestination = 0180C2000000
	false false FilterChainNew
	dup s" 66" ot_SAP32 ObjectParse ft_IEEE802_sap false FilterTestNew FilterChainAppend
	dup s" 0180C2000000" ot_MAC ObjectParse ft_MAC_dst false FilterTestNew FilterChainAppend
;

: runSTP
	initTspecSTP
        initTwalkSTP
            dup filterSTP IntervalWalkSetFileFilter
        runQuery ;

\ =================== data-center bridging discovery protocol query ===========================

: initTspecDCB { -- tspec }
        resetColumnIndex
	TableSpecNew
		dup ft_agent_IP	   	   addKey
		dup ft_ds_class	   	   addKey
		dup ft_ds_index	   	   addKey
		dup ft_IF_in	   	   addKey
		dup ft_IF_out	   	   addKey
		dup ft_MAC_src     	   addKey
		dup ft_MAC_dst     	   addKey
		dup -1 TableSpecSetN
;

: initTwalkDCB { tspec | -- twalk }
	tspec
	db_ACTIVEFLOWS db_UNDEFINED db_UNDEFINED db_UNDEFINED
	IntervalWalkNew

	dup 0 IntervalWalkNewGroup
	dup
	latestActiveFlows
	1 minute *                    \ 1-minute duration
	IntervalWalkAdd
;

: filterDCB
	\ filter to select only dcb traffic:  trillhops != null
	false false FilterChainNew
	dup s" null" ot_INT32 ObjectParse ft_TRILL_hops true FilterTestNew FilterChainAppend
;

: runDCB
	initTspecDCB
        initTwalkDCB
            dup filterDCB IntervalWalkSetFileFilter
        runQuery ;

\ =================== counters query ===========================

: addAttribute  ( spec field -- ) agg_FIRST sort_NONE addColumn ;

: initTspecCTR { -- tspec }
        resetColumnIndex
	TableSpecNew
		dup ft_agent_IP		   addKey
		dup ft_ifIndex		   addKey
		dup ft_ifStatus		   addAttribute
		dup ft_ifType		   addAttribute
		dup ft_ifSpeed	   	   addAttribute
		dup ft_ds_countersProtocol addAttribute
		dup ft_intervalIndex	   addDerived
		dup ft_intervalSeconds	   addDerived
		dup ft_counterUpdates	   addValue
		dup ft_ifInOctets  	   addValue
		dup ft_ifOutOctets     	   addValue
		dup ft_ifInUcastPkts  	   addValue
		dup ft_ifOutUcastPkts  	   addValue
		dup ft_ifInBroadcastPkts   addValue
		dup ft_ifOutBroadcastPkts  addValue
		dup ft_ifInMulticastPkts   addValue
		dup ft_ifOutMulticastPkts  addValue
		dup -1 TableSpecSetN
;

: initTwalkCTR { tspec | -- twalk }
	tspec
	db_ACTIVECOUNTERS db_UNDEFINED db_UNDEFINED db_UNDEFINED
	IntervalWalkNew
	\ add 30 x 10-minute groups,  with the last one ending 10 minutes ago
	10 310 do
	 dup
	 dup 0 IntervalWalkNewGroup
	  now minute start I minute * -     \ start
	  10 minute *                       \ duration
	  IntervalWalkAdd
        -10 +loop
;

: streamFilter { streamFile streamFileCount -- filter }
  resetColumnIndex
  TableSpecNew
  dup ft_stream_id addKey
  dup MapNew swap  \ ( map spec -- )
  nullString streamFile streamFileCount 0 nullString semicolonCommentChar nullString TableSpecParseFile  \ (map table -- )
  over TableAddTo \ ( map -- )
  ft_stream_id false FilterMapNew ; \ ( filter -- )
  
: runCTR { streamFile streamFileCount -- }
  initTspecCTR
  initTwalkCTR
   dup streamFile StreamFileCount streamFilter IntervalWalkSetFileFilter
  runQuery ;

\ =================== management server discovery ===========================
\ The purpose of this was to find top-layer addresses that were seen talking
\ everywhere (e.g. IP address of network management server, DNS server, web-proxy...)
\ and then list the ports that saw traffic to/from those addresses.  It's unlikely
\ to be used because it tends to light up leaf ports too,  even when filtering
\ to select only SNMP or sFlow traffic.

: initTspecMGT { -- tspec }
        resetColumnIndex
	TableSpecNew
		dup ft_dst     	           addKey
		dup ft_agent_IP	   	   agg_COUNT sort_DESCENDING addColumn
		dup 4 TableSpecSetN
		dup true TableSpecSuppressOther
;

: initTspecMGT_map
        resetColumnIndex
	TableSpecNew
		dup ft_dst addKey  \ just the key field
;

: initTwalkMGT { tspec | -- twalk }
	tspec
	db_ACTIVEFLOWS db_UNDEFINED db_UNDEFINED db_UNDEFINED
	IntervalWalkNew

	dup 0 IntervalWalkNewGroup
	dup
	latestActiveFlows
	30 minute * -
	30 minute *                    \ 30-minute duration
	IntervalWalkAdd
;

: filterMGT
	false false FilterChainNew \ AND chain
	\ only devices that are switching or routing
	dup
	      false true FilterChainNew \ OR chain
	      dup s" 1" ot_INT32 ObjectParse ft_hasFlows_SWITCH false FilterTestNew FilterChainAppend
	      dup s" 1" ot_INT32 ObjectParse ft_hasFlows_ROUTER false FilterTestNew FilterChainAppend
	FilterChainAppend
	\ only unicast traffic
	dup s" 1" ot_INT32 ObjectParse ft_isUnicast false FilterTestNew FilterChainAppend
	\ selecting UDP will focus on sFlow, NetFlow, DNS, bootp - but we might find that ICMP or TCP
	\ does just as well at illuminating paths
	\ dup s" UDP" ot_PROTOCOL ObjectParse ft_protocol false FilterTestNew FilterChainAppend
	\ filtering by service protocol is not really necessary
	\ dup
	\      false true FilterChainNew \ OR chain
	\      dup s" 161" ot_SAP32 ObjectParse ft_UDP_dst_port false FilterTestNew FilterChainAppend
	\      dup s" 162" ot_SAP32 ObjectParse ft_UDP_dst_port false FilterTestNew FilterChainAppend
	\      dup s" 6343" ot_SAP32 ObjectParse ft_UDP_dst_port false FilterTestNew FilterChainAppend
	\      dup s" 26343" ot_SAP32 ObjectParse ft_UDP_dst_port false FilterTestNew FilterChainAppend
	\      dup s" 9985" ot_SAP32 ObjectParse ft_UDP_dst_port false FilterTestNew FilterChainAppend
	\      dup s" 19985" ot_SAP32 ObjectParse ft_UDP_dst_port false FilterTestNew FilterChainAppend
	\      dup s" 8" ot_SAP32 ObjectParse ft_ICMP_type false FilterTestNew FilterChainAppend
	\      dup s" 0" ot_SAP32 ObjectParse ft_ICMP_type false FilterTestNew FilterChainAppend
	\      dup s" 8" ot_SAP32 ObjectParse ft_ICMP6_type false FilterTestNew FilterChainAppend
	\      dup s" 0" ot_SAP32 ObjectParse ft_ICMP6_type false FilterTestNew FilterChainAppend
	\ FilterChainAppend
;

: endGroupMGT { results | -- }
   \ use the results to make a filter
   results initTspecMGT_map MapNew dup >r TableAddTo \ (map>r)
   \ and apply it to a new query
   resetColumnIndex
   TableSpecNew
    dup ft_agent_IP addKey
    dup ft_ds_index addKey
    dup ft_MAC_src addKey
    dup -1 TableSpecSetN
   initTwalkMGT \ (twalk --)
   dup
   filterMGT
   dup r> ft_dst false FilterMapNew FilterChainAppend
   IntervalWalkSetFileFilter
   runQuery
   ;

: runQueryMGT { walker | -- } walker 0 0 ['] endGroupMGT IntervalWalkQueryHT ;

: runMGT
	initTspecMGT
        initTwalkMGT
            dup filterMGT IntervalWalkSetFileFilter
        runQueryMGT ;


\ =================== pervasive MAC discovery ===========================
\ Find the MACs that are seen in the most places.  These will typically be
\ router MACs,  or the MACs belonging to DNS or Network-Manager servers or
\ proxies. Interfaces that see these MACs on ingress are likely to be
\ inter-device-links.  It makes sense to do this separately for each VLAN.

: initTspecPMC { -- tspec }
        resetColumnIndex
	TableSpecNew
		dup ft_MAC_src		addKey
		dup ft_agent_IP	   	agg_COUNT sort_DESCENDING addColumn
		dup 4 TableSpecSetN
		dup true TableSpecSuppressOther
;

: initTspecPMC_map
        resetColumnIndex
	TableSpecNew
		dup ft_MAC_src addKey  \ just the key field
;

: initTwalkPMC { tspec | -- twalk }
	tspec
	db_ACTIVEFLOWS db_UNDEFINED db_UNDEFINED db_UNDEFINED
	IntervalWalkNew

	dup 0 IntervalWalkNewGroup
	dup
	latestActiveFlows
	30 minute * -
	30 minute *                    \ 30-minute duration
	IntervalWalkAdd
;

: filterPMC
	false false FilterChainNew \ AND chain
	\ only devices that are switching or routing
	dup
		false true FilterChainNew \ OR chain
		dup s" 1" ot_INT32 ObjectParse ft_hasFlows_SWITCH false FilterTestNew FilterChainAppend
		dup s" 1" ot_INT32 ObjectParse ft_hasFlows_ROUTER false FilterTestNew FilterChainAppend
	FilterChainAppend
	\ only unicast traffic
	dup
		s" 1" ot_INT32 ObjectParse ft_isUnicast false FilterTestNew
	FilterChainAppend
;

: endGroupPMC { results | -- }
   \ use the results to make a filter
   results initTspecPMC_map MapNew dup >r TableAddTo \ (map>r)
   \ and apply it to a new query
   resetColumnIndex
   TableSpecNew
    dup ft_agent_IP addKey
    dup ft_IF_in addKey
    dup -1 TableSpecSetN
   initTwalkPMC \ (twalk --)
   dup
   filterPMC
   dup r> ft_MAC_src false FilterMapNew FilterChainAppend
   IntervalWalkSetFileFilter
   runQuery
   ;

: runQueryPMC { walker | -- } walker 0 0 ['] endGroupPMC IntervalWalkQueryHT ;

: runPMC
	initTspecPMC
        initTwalkPMC
            dup filterPMC IntervalWalkSetFileFilter
        runQueryPMC ;


\ important to have a cr after the last ';' to make sure the last word gets compiled
