\ vocabularly for accessing raw packet data
load ../../dataDictionary/common/const.4th

\ relevant primitives are:

\ SampleHeader ( sample protocolType -- ptr_or_NULL )
\ SamplePayload ( sample protocolType -- ptr_or_NULL )
\ SamplePayloadBytes ( sample protocolType -- numBytes_or_0 )
\ SampleField ( sample field -- ptr_or_NULL )  \ if variable length, then len is first byte. This is not a pointer into the packet!
\ SampleMatchBytes ( sample regionStartPtr regionLen patternPtr patternLen maskPtr_or_NULL -- ptr_or_NULL )
\ SampleEnd ( sample -- ptr_or_NULL )

\ filters will take the form:
\ : filter_<nnn> { sample -- flag }
\ sample arg test if sample arg test if sample arg test then then ;

\
\ Utility functions
\
\ To put print statements in, use the line
\  dup ." var = " . cr

\ find if number is in range
: between { val min max -- flag }
 val min = 
 if true 
 else
  val max = 
   if true 
   else
    val min > val max < and
   then 
 then ;

\
\ Content matching functions
\

\ non-payload functions

\ IP header

\ test IP fragbits
: fragbits { sample value mask -- flag }
 sample pt_IP SampleHeader
 dup  
 if
  6 + c@
  mask and value =
 then ;

\ test IP frag offset
: fragoffset { sample min max -- flag } 
 sample pt_IP SampleHeader 
 dup 
 if
  6 + q@ ntoh32 8191 and       \ mask the lower 13 bits
  min max between
 then ;

\ test IP TTL
: ttl { sample min max -- flag }
 sample pt_IP SampleHeader
 dup
 if
  8 + c@
  min max between
 then ;

\ test IP TOS
: tos { sample value -- flag }
 sample pt_IP SampleHeader
 dup
 if
  1 + c@
  value =
 then ;

\ test IP ID
: id { sample value -- flag }
 sample pt_IP SampleHeader
 dup 
 if
  4 + q@ ntoh32
  value =
 then ;

\ test IP protocol
: ip_proto { sample min max -- flag }
 sample pt_IP SampleHeader
 dup 
 if
  9 + c@
  min max between
 then ;

\ test IP options
\ : ipopts { sample option -- flag }

\ test if source and destination addresses are same
: sameip { sample | src dst -- flag }
 sample ft_IP_src SampleField to src
 sample ft_IP_dst SampleField to dst
 src 0> dst 0>  and
 if
  src 4 dst 4 compare 0=
 else false
 then ;

\ ICMP header

\ test ICMP type
: itype { sample min max -- flag }
 sample pt_ICMP SampleHeader
 dup 
 if
  c@
  min max between
 then ;

\ test ICMP code
: icode { sample min max -- flag }
 sample pt_ICMP SampleHeader
 dup 
 if
  1 + c@
  min max between
 then ;

\ test ICMP id
: icmp_id { sample value -- flag }
 sample pt_ICMP SampleHeader
 dup 
 if
  4 + q@ ntoh32
  value =
 then ;

\ test ICMP sequence number
: icmp_seq { sample value -- flag }
 sample pt_ICMP SampleHeader
 dup 
 if
  6 + q@ ntoh32
  value =
 then ;

\ TCP header
                                                                                
\ test TCP flags
: flags { sample value mask -- flag }
 sample pt_TCP SampleHeader
 dup 
 if 
  13 + c@
  mask and value = 
 then ; 

\ test TCP sequence no.
: seq { sample value -- flag }
 sample pt_TCP SampleHeader
 dup 
 if
  4 + dup                   \ get offset of start of seqno and save it
  q@ ntoh32                  \ read the 32-bit integer (and swap to host byte order)
  value =
 then ;

\ test TCP ack
: ack { sample value -- flag }
 sample pt_TCP SampleHeader
 dup 
 if
  8 + dup
  q@ ntoh32
  value =
 then ;

\ test TCP window
: window { sample value -- flag }
 sample pt_TCP SampleHeader
 dup 
 if
  14 + q@ ntoh32
  value =
 then ;

\ Payload

\ test data size
: dsize { sample protocol min max  -- flag }
 sample protocol SamplePayloadBytes \ zero could mean error or no bytes
 min max between ;                  \ is it in the range ?

\ define global variables for content matching
variable data_start
variable data_cursor

\ initialize cursor
: initcursor { sample protocol -- flag }
 sample protocol SamplePayload
 dup
 if
  dup
  data_start !
  data_cursor !
  true
 then ;

\ return current cursor postion as an offset
: offset ( -- offset )
  data_cursor @
  data_start @
  - ;

\ test packet contents
: content { sample offset within pattern patternLen mask -- flag }
 sample 
 data_start @ offset +
 within 
 pattern 
 patternLen 
 mask
 SampleMatchBytes
 dup
 if
   dup patternLen + data_cursor !
 then ;

\ match GET<sp>, then match content
\ should urldecode content
create p_http_get 71 c, 69 c, 84 c, 32 c,
: uricontent { sample offset within pattern patternLen mask -- flag }
 sample data_start @ 4 p_http_get 4 0 SampleMatchBytes
 dup
 if
  4 + data_cursor !
  sample offset within pattern patternLen mask content
 then ;

\ test if fields exist
: isdataat { sample protocol offset -- flag }
 sample protocol SamplePayloadBytes
 dup
 if
   offset 
   < 0= 
   dup
   if
     data_start @ offset + data_cursor ! 
   then
 then ;

variable ival

\ get big endian integer
: getintb { sample offset len -- flag }
  sample SampleEnd
  dup
  if
    data_start @ offset +
    dup data_cursor !
    len +
    >
    dup
    if
      0
      len 0 do
        data_cursor @ i +
        c@
        swap 8 lshift +
      loop
      ival !
      data_cursor @ len + data_cursor ! 
    then
  then ; 

\ get little endian integer
: getintl { sample offset len -- flag }
  sample SampleEnd
  dup
  if
    data_start @ offset +
    dup data_cursor !
    len +
    >
    dup
    if
      0
      len 1 + 1 do
        data_cursor @ len i - +
        c@
        swap 8 lshift +
      loop
      ival !
      data_cursor @ len + data_cursor !
    then
  then ;

\ convert a character into an integer value, -1 if bad character
: digitval ( base char -- int ) 
  dup 47 > 
  if dup 58 <
    if 48 - 
    else dup 64 >
      if dup 71 <
        if 55 -
        else dup 96 >
          if dup 103 <
            if 87 -
            else drop -1
            then 
          else drop -1
          then
        then
      else drop -1
      then
    then
  else drop -1
  then
  dup rot >
  if drop -1
  then ;
  
: getinta { sample offset len base -- flag }
  sample SampleEnd
  dup
  if
    data_start @ offset +
    dup data_cursor !
    len +
    >
    dup
    if
      0
      len 0 do
        data_cursor @ i +
        c@
        dup 32 =
        if drop
        else
          base swap digitval
          dup 0<
          if drop swap drop false swap leave
          else swap base * +
          then
        then
      loop
      ival !
      data_cursor @ len + data_cursor !
    then
  then ;

: align ( offset -- offset )
  dup 4 mod dup 0= if drop else 4 swap - + then ; 
