String.prototype.trim = function() {
  return this.replace(/^\s+/g, '').replace(/\s+$/g, '');
}

function Tok(type, value) {
  this.type = type;
  this.value = value;
}

Tok.NOT        = "!";
Tok.LBRACKET   = "(";
Tok.NLBRACKET  = Tok.NOT + Tok.LBRACKET;
Tok.RBRACKET   = ")";
Tok.EQ         = "=";
Tok.NEQ        = Tok.NOT + Tok.EQ;
Tok.MATCH      = "~";
Tok.NMATCH     = Tok.NOT + Tok.MATCH;
Tok.OR         = "|";
Tok.AND        = "&";

Tok.TBRACKET   = 1;
Tok.TCMP       = 2;
Tok.TBOOL      = 3;

Tok.prototype.toString = function() {
  return this.value;
}

function Filter(initial) {
  this.tokens = new Array();
  this.state = 0;
  this.depth = 0;
  this.chains = new Array();
  if(initial) {
    var parts = initial.split("_$DELIM$_");
    var tokStr = "";
    var tok = null;
    for(var i = 0; i < parts.length; i++) {
       tokStr += parts[i];
       if("!(" == tokStr) tok = new Tok(Tok.TBRACKET,Tok.NLBRACKET);
       else if("(" == tokStr) tok = new Tok(Tok.TBRACKET,Tok.LBRACKET);
       else if(")" == tokStr) tok = new Tok(Tok.TBRACKET,Tok.RBRACKET);
       else if("&" == tokStr) tok = new Tok(Tok.TBOOL,Tok.AND);
       else if("|" == tokStr) tok = new Tok(Tok.TBOOL,Tok.OR);
       else tok = new Tok(Tok.TCMP,tokStr);
       if(tok) {
         this.addToken(tok);
         tokStr = "";
      }
    }
  }
}

//Filter.toggleControls = function(id)
//{
//  var style = document.getElementById(id).style;
//  style.display = style.display  == "block" ? "none":"block";
//}

Filter.toggleControls = function(id)
{
  $("#" + id).slideToggle("fast");
}

Filter.scanString = function(string) {
  return new Tok(Tok.TBRACKET,"test this");
}

Filter.filterForm = function(form) {
  return document.forms[form];
}

Filter.displayFilter = function(form,field) {
  var filt = this.current(form,field);
  if(filt.inputform) {
     var fin = this.filterForm(filt.inputform);
     fin[field].value = filt.toString();
  }
  else {
     var fin = this.filterForm(form);
     fin[field].value = filt.toString();
  }
  this.setKeys(form,field);
  var f = this.filterForm(form);
  f[field + "_val"].style.background = "#ffffff";
  f[field + "_val"].value = "";
}

Filter.setDisabled = function(name,flag) {
  var node = document.getElementsByName(name)[0];
  if(node) {
     node.disabled = flag;
     if(navigator.userAgent.match("Safari") && node.type == "button") {
       if(flag) {
         node.style.color = "#666";
       }
       else {
         node.style.color = "#000";
       }
     }
  }
}

Filter.setKeys = function(form,field) {
  var filt = this.current(form,field);
  var tok = filt.lastToken(); 
  if(tok == null) {
     this.setDisabled(field + "_col", false);
     this.setDisabled(field + "_cmp", false);
     this.setDisabled(field + "_val", false);
     this.setDisabled(field + "_ADDVAL", false);
     this.setDisabled(field + "_AND", true);
     this.setDisabled(field + "_OR", true);
     this.setDisabled(field + "_LBRACKET", false);
     this.setDisabled(field + "_NLBRACKET", false);
     this.setDisabled(field + "_RBRACKET", true);
     this.setDisabled(field + "_DELETE", true);
     this.setDisabled(field + "_CLEAR", true);
  }
  else {
     this.setDisabled(field + "_DELETE", false);
     this.setDisabled(field + "_CLEAR", false);

     if(Tok.TCMP == tok.type) {
       this.setDisabled(field + "_col", true);
       this.setDisabled(field + "_cmp", true);
       this.setDisabled(field + "_val", true);
       this.setDisabled(field + "_ADDVAL", true);
      
       this.setDisabled(field + "_AND", false);
       this.setDisabled(field + "_OR", false);

       this.setDisabled(field + "_LBRACKET", true);
       this.setDisabled(field + "_NLBRACKET", true);

       if(filt.depth > 0) {
          this.setDisabled(field + "_RBRACKET", false);
       }
     }
     else {
       this.setDisabled(field + "_col", false);
       this.setDisabled(field + "_cmp", false);
       this.setDisabled(field + "_val", false);
       this.setDisabled(field + "_ADDVAL", false);

       if(Tok.TBOOL == tok.type) {
         this.setDisabled(field + "_AND", true);
         this.setDisabled(field + "_OR", true);
  
         this.setDisabled(field + "_LBRACKET", false);
         this.setDisabled(field + "_NLBRACKET", false);
         this.setDisabled(field + "_RBRACKET", true);
       }
       else {
         if(Tok.TBRACKET == tok.type && Tok.RBRACKET == tok.value) {
           this.setDisabled(field + "_AND", false);
           this.setDisabled(field + "_OR", false);
         }
         else {
           this.setDisabled(field + "_AND", true);
           this.setDisabled(field + "_OR", true);
         }

         if(filt.depth == 0) {
           this.setDisabled(field + "_RBRACKET", true);
         }
       }
     }
  }
}

Filter.current = function(form,field) {
   var filter = this[form + "." + field];
   if(!filter) {
      // should initialize filter to current value of field
      var f = this.filterForm(form);
      filter = new Filter(f[field + "_tokens"].value);
      this[form + "." + field] = filter;
   }
   return filter;
}

Filter.clear = function(form,field) {
   var oldFilter = this[form + "." + field];
   var newFilter = new Filter();
   if(oldFilter.inputform) newFilter.inputform = oldFilter.inputform;
   this[form + "." + field] = newFilter;
}

Filter.ipv4pat = "([\\d]{1,3}\\.){3}[\\d]{1,3}";
Filter.ipv6pat = "([0-9:A-Fa-f])*:([0-9:A-Fa-f])*";
Filter.macpat = "[\\dA-Fa-f]{12}";
Filter.appletalkpat = "^[0-9]+\\.[0-9]+$";
Filter.dec4pat = "^D[0-9]+\\.[0-9]+$";
Filter.ipxpat = "^" + Filter.ipv4pat + "[ ]" + Filter.macPat + "$";

Filter.addresstype = function(address) {
  var type = null;
  if(address.match("^" + this.ipv4pat + "$")) type = "ipv4";
  else if(address.match("^" + this.macpat + "$")) type = "mac";
  else if(address.match("^" + this.ipv6pat + "$")) type = "ipv6";
  else if(address.match(this.ipxpat)) type = "ipx";
  else if(address.match(this.appletalkpat)) type = "appletalk";
  else if(address.match(this.dec4pat)) type = "dec4";
  return type; 
}

Filter.validate = function(type,value) {
  if(!value) return false;

  var vlist = value.split(",");

  var result = true;

  var i = 0;

  do {
    if("address" == type) {
      var atype = this.addresstype(vlist[i]);
      result = atype != null;
    }
    else if("integer" == type) {
      result = vlist[i].match("^[0-9]+$");
    }
    else if("string" == type) {
      result = true;
    }
    else if("url" == type) {
      result = true;
    }
    else if("path" == type) {
      result = true;
    }
    else if("port" == type) {
      result = true;
    } 
    else if("as" == type) {
      result = true;
    }
    else if("agent" == type) {
      result = true;
    }
    else if("interface" == type) {
       result = true;
    }
    else if("protocol" == type) {
       result = true;
    }
  } while(result && ++i < vlist.length);

  return result;
}

Filter.prototype.removeToken = function() {
  this.tokens.pop();
}

Filter.prototype.lastToken = function() {
  return this.tokens[this.tokens.length - 1];
}

Filter.prototype.addToken = function(tok) {
  if(Tok.TBOOL == tok.type) {
    if(this.chains[this.depth] == null) {
      this.chains[this.depth] = tok.value;
    }
    else if(this.chains[this.depth] != tok.value) {
      var prev = this.lastToken();
      if(Tok.TBRACKET == prev.type && Tok.RBRACKET == prev.value) {
        // search backward to matching open bracket
        var idx = this.tokens.length - 1;
        var d = 1;
        while(idx >= 0 && d > 0) {
          var t = this.tokens[--idx];
          if(Tok.TBRACKET == t.type) {
            if(Tok.RBRACKET == t.value) d++;
            else d--;
          }
        }
        this.tokens.splice(idx,0,new Tok(Tok.TBRACKET,Tok.LBRACKET));
        this.chains[++this.depth] = tok.value;
      }
      else {
        prev = this.tokens.pop();
        this.tokens.push(new Tok(Tok.TBRACKET,Tok.LBRACKET));
        this.chains[++this.depth] = tok.value;
        this.tokens.push(prev); 
      }
    }
  }
  else if(Tok.TBRACKET == tok.type) {
    if(Tok.LBRACKET == tok.value || Tok.NLBRACKET == tok.value) {
      this.chains[++this.depth] = null;
    }
    else if(Tok.RBRACKET == tok.value) {
      this.depth--;
    }
  }
  this.tokens.push(tok);
}

Filter.prototype.toString = function() {
  var result = null;

  if(this.tokens.length == 0) result = "";
  else {
    for(var i = 0; i < this.tokens.length; i++) {
      if(result) result += " " + this.tokens[i].toString();
      else result = this.tokens[i].toString();
    }
  }
  return result;
}

Filter.addToken = function(form,field,tok) {
  this.current(form,field).addToken(tok);
  this.displayFilter(form,field);
}

Filter.keyCLEAR = function(form,field) {
  this.clear(form,field);
  this.displayFilter(form,field);
}

Filter.keyDELETE = function(form,field) {
  this.current(form,field).removeToken();
  this.displayFilter(form,field);
}

Filter.keyLBRACKET = function(form,field) {
  this.addToken(form,field,new Tok(Tok.TBRACKET, Tok.LBRACKET));
}

Filter.keyNLBRACKET = function(form,field) {
  this.addToken(form,field,new Tok(Tok.TBRACKET, Tok.NLBRACKET));
}

Filter.keyRBRACKET = function(form,field) {
  this.addToken(form,field,new Tok(Tok.TBRACKET, Tok.RBRACKET));
}

Filter.keyAND = function(form,field) {
  this.addToken(form,field,new Tok(Tok.TBOOL, Tok.AND));
}

Filter.keyOR = function(form,field) {
  this.addToken(form,field,new Tok(Tok.TBOOL, Tok.OR));
}

Filter.keyADDVAL = function(form,field) {
  var f = this.filterForm(form);

  var col = f[field + "_col"].options[f[field + "_col"].selectedIndex].value;
  var cmp = f[field + "_cmp"].options[f[field + "_cmp"].selectedIndex].value;
  var val = f[field + "_val"].value.trim();

  var parts = col.split(".");

  var cid = parts[0];
  var ctype = parts[1]; 

  var valid = this.validate(ctype,val);

  if(valid) this.addToken(form,field,new Tok(Tok.TCMP, cid + " " + cmp + " " + val));
  else f[field + "_val"].style.background = "#ffcccc";
}

Filter.filterlist = new Array();

Filter.initializeFilter = function(form,field,inputform,url) {
   if(inputform) this.filterlist.push(new Array(form,field,inputform,url));
   else this.filterlist.push(new Array(form,field));
}

$(document).ready(function() {
   if(Filter.filterlist) {
     for(var i = 0; i < Filter.filterlist.length; i++) {
       var entry = Filter.filterlist[i];
       var filt = Filter.current(entry[0],entry[1]);
       if(entry.length <= 4) {
         filt.inputform = entry[2];
         filt.url = entry[3];
  
         $("#" + entry[1]).keypress(function(event) {
            var flag = event.keyCode == 13; 
            if(flag) location.href= entry[3] + encodeURIComponent($(this).val());
            return !flag;
          });
          $("#" + entry[1] + "_ok").click(function() {
            location.href = entry[3] + encodeURIComponent($("#" + entry[1]).val());
          });
          $("#" + entry[1] + "_clear").click(function() {
            location.href = entry[3];
          });
          $("#" + entry[1] + "_edit").click(function() {
            Filter.toggleControls(entry[0]);
          });
       };
       $("#" + entry[1] + "_val").keypress(function(event) {
         return event.keyCode != 13;
       });
       $("#" + entry[1] + "_ADDVAL").click(function() {
         Filter.keyADDVAL(entry[0],entry[1]);
       });
       $("#" + entry[1] + "_AND").click(function() {
         Filter.keyAND(entry[0],entry[1]);
       });
       $("#" + entry[1] + "_OR").click(function() {
         Filter.keyOR(entry[0],entry[1]);
       });
       $("#" + entry[1] + "_LBRACKET").click(function() {
         Filter.keyLBRACKET(entry[0],entry[1]);
       });
       $("#" + entry[1] + "_NLBRACKET").click(function() {
         Filter.keyNLBRACKET(entry[0],entry[1]);
       });
       $("#" + entry[1] + "_RBRACKET").click(function() {
         Filter.keyRBRACKET(entry[0],entry[1]);
       });
       $("#" + entry[1] + "_DELETE").click(function() {
         Filter.keyDELETE(entry[0],entry[1]);
       });
       $("#" + entry[1] + "_CLEAR").click(function() {
         Filter.keyCLEAR(entry[0],entry[1]);
       });         
       
       Filter.setKeys(entry[0],entry[1]);
     }
   }
});
