// Copyright (c) 2012 InMon Corp. ALL RIGHTS RESERVED

(function($) {
  $.fn.charts = function(settings) {
    
    var defaults = {
        height         : 140,
        width          : 500,
        largeHeight    : 300,
        columns        : 2,
        legendRows     : 1,
        refresh        : false,
        rebuild        : true,
        large          : false,
        step           : true,
        dataTimeout    : 600000
    };
    var option = $.extend(defaults,settings);

    var myID = $(this).attr('id');
    var myDiv = $(this);
    var myData;

    if(option.dataURL) {
         getData();
    }

    function removeLoading() {
      $("#loading").empty().hide();
      $("#content").css("min-height","0");
      return true;
    }

    var loadingRemoved = false;
    function getData() {
      $.ajax({
        url: option.dataURL, 
        success: function(data) {
          if(!loadingRemoved) { loadingRemoved = removeLoading(); }
          drawCharts(data.data);
          if(option.refresh) setTimeout(getData, 60000);    
        },
        error: function(result,status,errorThrown) {
             myDiv.empty();
             $("#loading").empty().append('<span class="warn">' + status + (errorThrown ? '&nbsp;' + errorThrown : '') + '</span>').show();
        },
        timeout: parseInt(option.dataTimeout)
      });
    };

    function processResponse(response,data) {
      if(!response) return;

      switch(response.action) {
        case "loadpage":
          if(response.url) window.location = response.url;
          break;
        default:
      }
    }

    function chartEventHandler(event, gridpos, datapos, neighbor, plot) {
      var nodeClick = $(this).data("nodeClick");
      if(nodeClick) {
        $.getJSON(option.dataURL + "&_evtName=nodeClick&_evtId=" + escape(nodeClick.id) + "&_evtValue=" + escape(nodeClick.value ? nodeClick.value : ""), function(response) {
          processResponse(response,nodeClick);
        });
      }
    }

    var initialized = false;

    function drawCharts(data) {
      myData = data;
      if(!initialized || option.rebuild) {
         initCharts(data);
         initialized = true;
      }
      updateCharts(data);
    }

    function initCharts(data) {
      var ncols = option.columns;
      var i, charts, chart, opts, lheight, h = option.height, w = option.width;
      if(ncols > 0) {
        charts = '<table><tbody><tr>';
        for(i = 0; i < data.length; i++) {
          chart = data[i];
          if(i > 0 && i % ncols == 0) charts += '</tr><tr>';
          charts += '<td class="padChart">';
          if(chart.title) charts += '<h2>'+chart.title+'</h2>';
          charts += '<div id="chart'+i+'" class="trend"></div></td>';
        };
        charts += '</tr></tbody></table>';
      } else {
        charts = ''
        if(option.large && data.length == 1) {
          chart = data[0];
          w = "100%";
          h = option.largeHeight;
          if(chart.title) charts += '<h2>'+chart.title+'</h2>';
          charts += '<div id="chart0" class="trend"></div>';
        } else {
          for(i = 0; i < data.length; i++) {
            chart = data[i];
            charts += '<div class="floatChart">';
            if(chart.title) charts += '<h2>'+chart.title+'</h2>';
            charts += '<div id="chart'+i+'" class="trend"></div>';
            charts += '</div>';
          }
          charts += '<div class="clearChart"></div>';
        }
      }
      $('stripchart').stripchart("destroy");
      myDiv.empty();
      myDiv.append(charts);
      lheight = h;
      for(i = 0; i < data.length; i++) {
        chart = data[i];
        if(chart.yaxis.series.length > 6) {
          lheight = Math.floor(Math.max(lheight,option.height * (chart.yaxis.series.length / 6)));
        }
      }
      for(i = 0; i < data.length; i++) {
        chart = data[i];
        opts = {
          stack:chart.stack || chart.mirror,
          mirror:chart.mirror,
          step:option.step,
          includeMillis:chart.includeMillis
        }; 
        if(chart.colors) opts.colors = chart.colors.split(",");
        $('#chart' + i).stripchart(opts).css("height",lheight).css("width",w);
        if(chart.nodeClick) {
          $('#chart' + i).data("nodeClick", chart.nodeClick)
                         .bind("click",chartEventHandler)
                         .css("cursor","pointer");
        }
      }
    }

    function updateCharts(data) {
      for(var i = 0; i < data.length; i++) {
	var chart = data[i];
        var cdata = {}; 
 	cdata.units = chart.yaxis.label;
	cdata.times = $.map(chart.xaxis.data,function(val) { return (new Date(val)).getTime(); });
        cdata.values = [];
        cdata.legend = {labels:[]};
        for(var s = 0; s < chart.yaxis.series.length; s++) {
	  cdata.values.push(chart.yaxis.series[s].values);
          cdata.legend.labels.push(chart.yaxis.series[s].name);
        }
        $('#chart' + i).stripchart('draw', cdata);
      }
    }
    $(window).resize(function() {
     if(myData) updateCharts(myData);
    });

    return this;
  }
})(jQuery);
