// Copyright (c) 2011 InMon Corp. ALL RIGHTS RESERVED

(function($) {
  $.fn.circles = function(settings) {
    
    var defaults = {
        height         : 200,
        bottomInset    : 0,
        leftInset      : 0,
        topInset       : 10,
        rightInset     : 0,
        refresh        : false,
        select         : "Select",
        exclude        : "Exclude", 
        search         : "Search",
        autolabel      : "none",
        colors         : "red,orange,yellow,green,blue,violet",
        dataTimeout    : 600000
    };
    var option = $.extend(defaults,settings);

    var clusterBoxWidth = 6;
    var nodeBoxWidth    = 6;
    var maxWidth        = 10;
    var cellsPerRow     = 4;

    var myID = $(this).attr('id');
    var myDiv = $(this);
    var myTable;
    var myCanvas;
    var myData;
    var colors;

    if(option.dataURL) {
         getData();
    }

   function getData() {
     $.ajax({
        url: option.dataURL + (myData ? "&refresh=true" : ""), 
        success: function(data) {
          myData = data;
          drawChart();
          if(option.refresh) setTimeout(getData, 60000);    
        },
        error: function(result,status,errorThrown) {
             myDiv.empty();
             myDiv.append('<span class="warn">' + status + (errorThrown ? '&nbsp;/&nbsp;' + errorThrown : '') + '</span>');
        },
        timeout: option.dataTimeout
     });
   };

   function drawChart() {
      if(!myTable) initChart();

      var canvas = myCanvas[0];
      if(!(canvas && canvas.getContext)) return;

      var ctx = canvas.getContext('2d');

      var h = myCanvas.height();
      var w = myCanvas.width();

      var ratio = window.devicePixelRatio;
      if(ratio && ratio > 1) {
           canvas.height = h * ratio;
           canvas.width = w * ratio;
           ctx.scale(ratio,ratio);
       }
       else {
           canvas.height = h;
           canvas.width = w;
       }

      updateLegend(myData.protocols); 
      layoutCircles();
      drawCircles(ctx,h,w);	
   }

   var nodeLocations;
   var clusterLocations;
   var maxVal = 0;
   var minVal = -1;
   function layoutCircles() {
      nodeLocations = {};
      clusterLocations = {};

      var flows = myData.flows;
      if(!flows) return;

      var clusterNodes = {};
      function addNodeToCluster(cluster,node) {
         var nodes = clusterNodes[cluster];
         if(!nodes) {
            nodes = {};
            clusterNodes[cluster] = nodes;
         }
         nodes[node] = node;
      }

      for(var i = 0; i < flows.length; i++) {
         var flow = flows[i];
         var sourcegroup = flow[0];
         var sourcenode = flow[1];
         var destinationgroup = flow[2];
         var destinationnode = flow[3];
         var protocol = flow[4];
         var val = flow[5];
         addNodeToCluster(sourcegroup,sourcenode);
         addNodeToCluster(destinationgroup,destinationnode);
         if(val > maxVal) maxVal = val;
         if(minVal == -1 || val < minVal) minVal = val;
      }
      
      var clusterSizes = {};
      var sortedClusters = [];
      var maxClusterSize = 0;
      var totalNodes = 0;
      for(var cluster in clusterNodes) {
         var nodes = clusterNodes[cluster];
         var n = 0;
         for(node in nodes) n++;
         clusterSizes[cluster] = n;
         sortedClusters.push(cluster);
         totalNodes += n;
         if(n > maxClusterSize) maxClusterSize = n;
      }
      sortedClusters.sort(function(a,b) { return (clusterSizes[b] - clusterSizes[a]) });

      function clusterRadius(siz, spacing) { return (spacing * siz) / (2 * Math.PI) }

      function layoutCluster(cluster,radius) {
         var pos = clusterLocations[cluster];
         var mid_x = pos[0];
         var mid_y = pos[1];
         var ndAngleInc = (2 * Math.PI) / Math.max(clusterSizes[cluster],1);
         var ndAngle = 0.0;
         var nodes = clusterNodes[cluster];
         for(var node in nodes) {
            var x = radius * Math.cos(ndAngle);
            var y = radius * Math.sin(ndAngle);
            nodeLocations[node] = [mid_x + x, mid_y + y];
            ndAngle += ndAngleInc;
         }
      }
      
      if(sortedClusters.length == 1) {
         clusterLocations[sortedClusters[0]] = [0.5, 0.5];
         layoutCluster(sortedClusters[0],0.4);
         return;
      }
      
      var nodeSpacing = 0.04;
      var netRadius = 0.25;
      var nodeSpacingStep = nodeSpacing / 15;
      var netRadiusStep = netRadius / 15;
      var clusterSpacingAngle = 0.0;
      var fit = false;
      var iterations = 0;
      while(!fit) {
         iterations++;
         var maxRadius = 0.0;
         var clusterAngleSum = 0.0;
         for(var i = 0; i < sortedClusters.length; i++) {
            var radius = clusterRadius(clusterSizes[sortedClusters[i]],nodeSpacing);
            if(radius > maxRadius) maxRadius = radius;
            var clusterAngle = 2.0 * Math.asin(radius / netRadius);
            clusterAngleSum += clusterAngle;
         }
         clusterSpacingAngle = (2.0 * Math.PI - clusterAngleSum) / sortedClusters.length;
         if((netRadius + maxRadius) > 0.48) nodeSpacing -= nodeSpacingStep;
         else if(clusterSpacingAngle < 0.01) {
           if((netRadius + maxRadius) < 0.4) netRadius += netRadiusStep;
           else nodeSpacing -= nodeSpacingStep;
         }
         else fit = true;
      }

      var lastClusterAngle = 0.0;
      var theta = 0.0;
      for(var i = 0; i < sortedClusters.length; i++) {
         var siz = clusterSizes[sortedClusters[i]];
         var radius = clusterRadius(siz,nodeSpacing);
         var clusterAngle = 2.0 * Math.asin(radius / netRadius);
         theta += clusterSpacingAngle + ((lastClusterAngle + clusterAngle) / 2.0);
         var x = 0.5 + (netRadius * Math.cos(theta));
         var y = 0.5 + (netRadius * Math.sin(theta));
         clusterLocations[sortedClusters[i]] = [x,y];
         lastClusterAngle = clusterAngle;
         layoutCluster(sortedClusters[i],radius);
      }     
   }

   function pointDist(x1, y1, x2, y2) {
        var vx = x2 - x1;
        var vy = y2 - y1;
        return Math.sqrt((vx * vx) + (vy * vy));
   }

   function nearestPointOnLine(x, y, x1, y1, x2, y2) {
      // from (px, py) return nearest point on line through (x1, y1) and (x2, y2)
      var dX = x2 - x1;
      var dY = y2 - y1;
      if(dX == 0) {
        if(dY == 0) return [x1, y1];
        else return [x1, y];
      }
      else if(dY == 0) return [x, y1];
      var dX2 = dX * dX;
      var dY2 = dY * dY;
      var newX = ((x * dX2) + (x2 * dY2) + ((y - y2) * dX * dY)) / (dX2 + dY2);
      var newY = (((x - newX) * dX) / dY) + y;
      if(newX >= Math.min(x1,x2)
         && newX <= Math.max(x1,x2)
         && newY >= Math.min(y1,y2)
         && newY <= Math.max(y1,y2)) return [newX, newY];
      var dist1 = pointDist(x,y,x1,y1);
      var dist2 = pointDist(x,y,x2,y2);
      if(dist1 <= dist2) return [x1,y1];
      else return [x2,y2];
   }

   function pointDistLine(x, y, x1, y1, x2, y2) {
      // from (px, py) return nearest point on line through (x1, y1) and (x2, y2)
      var p = nearestPointOnLine(x,y,x1,y1,x2,y2);
      return pointDist(x,y,p[0],p[1]);
   }

   function drawClusters(ctx,h,w) {
      ctx.strokeStyle = "#000";
      ctx.lineWidth = 1;
      var halfWidth = clusterBoxWidth / 2;
      for(var cluster in clusterLocations) {
         var loc = clusterLocations[cluster];
         var x = Math.floor(loc[0] * w) + 0.5;
         var y = Math.floor(loc[1] * h) + 0.5;
         ctx.strokeRect(x - halfWidth,y - halfWidth,clusterBoxWidth,clusterBoxWidth);
      }
   }

   function drawNodes(ctx,h,w) {
      ctx.fillStyle = "#000";
      ctx.lineWidth = 1;
      halfWidth = nodeBoxWidth / 2;
      for(var node in nodeLocations) {
         var loc = nodeLocations[node];
         var x = Math.floor(loc[0] * w) + 0.5;
         var y = Math.floor(loc[1] * h) + 0.5;
         ctx.fillRect(x - halfWidth,y - halfWidth,nodeBoxWidth,nodeBoxWidth);     
      }
   }

   function drawFlows(ctx,h,w) {
      var flows = myData.flows;
      if(!flows) return;

      // collect all connections between each node pair
      var edges = {};
      for(var i = 0; i < flows.length; i++) {
         var flow = flows[i];
         var sourcenode = flow[1];
         var destinationnode = flow[3];
         var protocol = flow[4];
         var val = flow[5];
         var key;
         if(sourcenode < destinationnode) key = sourcenode + "," + destinationnode;
         else key = destinationnode + "," + sourcenode;
         var edge = edges[key];
         if(!edge) {
            edge = [];
            edges[key] = edge;
         }
         edge.push([protocol,val]);
      }
     
      var edgeRange = maxVal - minVal; 
      function edgeWidth(val) {
         if(edgeRange == 0) return 2;
         var fract = (val - minVal) / edgeRange;
         var width = Math.round(maxWidth * fract);
         return width <= 0 ? 1 : width;
      }
      for(var key in edges) {
         var nodepair = key.split(",");
         var vals = edges[key];

         var sourcenode = nodepair[0];
         var destinationnode = nodepair[1];
         var source_pos = nodeLocations[sourcenode];
         var destination_pos = nodeLocations[destinationnode];

         var startx = source_pos[0] * w;
         var starty = source_pos[1] * h;
         var endx = destination_pos[0] * w;
         var endy = destination_pos[1] * h;
         var lx = endx - startx;
         var ly = endy - starty;
         if(lx == 0 && ly == 0) continue;

         var edgew = 0;
         for(var i = 0; i < vals.length; i++) edgew += edgeWidth(vals[i][1]);
         var r = Math.sqrt((lx * lx) + (ly * ly));
         var ux = lx / r;
         var uy = ly / r;
         var unx = 0 - uy;
         var uny = ux;
         var shiftx = unx * edgew / 2.0;
         var shifty = uny * edgew / 2.0;

         for(var i = 0; i < vals.length; i++) {
            var flow = vals[i];

            var protocol = flow[0];
            var val = flow[1];

            var subw = edgeWidth(val);
            var hwnx = unx * subw / 2.0;
            var hwny = uny * subw / 2.0;
            shiftx -= hwnx;
            shifty -= hwny;
            var ishiftx = Math.round(shiftx);
            var ishifty = Math.round(shifty);
            var taperx = Math.round(ux * r * 0.1);
            var tapery = Math.round(uy * r * 0.1);
            var vertex1x = startx + ishiftx + taperx;
            var vertex1y = starty + ishifty + tapery;
            var vertex2x = endx + ishiftx - taperx;
            var vertex2y = endy + ishifty - tapery;
            var ihwnx = Math.round(hwnx);
            var ihwny = Math.round(hwny);
            var color = colors[protocol % colors.length];
            ctx.fillStyle=color;
            ctx.strokeStyle=color;
            ctx.lineWidth = 1;
            ctx.beginPath();
            ctx.moveTo(startx,starty);
            ctx.lineTo(vertex1x,vertex1y);
            ctx.lineTo(vertex2x,vertex2y);
            ctx.lineTo(endx,endy);
            ctx.stroke();
            ctx.closePath();
            ctx.beginPath();
            ctx.moveTo(vertex1x + ihwnx,vertex1y + ihwny);
            ctx.lineTo(startx,starty);
            ctx.lineTo(vertex1x - ihwnx,vertex1y - ihwny);
            ctx.lineTo(vertex2x - ihwnx,vertex2y - ihwny);
            ctx.lineTo(endx,endy);
            ctx.lineTo(vertex2x + ihwnx,vertex2y + ihwny);
            ctx.fill();
            ctx.closePath();

            shiftx -= hwnx;
            shifty -= hwny;
         }
      }     
   }

   function drawLabel(ctx,x,y,label) {
      ctx.fillStyle = "#000";
      var meas = ctx.measureText(label);
      ctx.strokeStyle = "#000";
      ctx.fillStyle = "#fff";
      var radius = 2;
      var width = meas.width + 4;
      var height = 16;
      ctx.beginPath();
      ctx.moveTo(x + radius, y);
      ctx.lineTo(x + width - radius, y);
      ctx.quadraticCurveTo(x + width, y, x + width, y + radius);
      ctx.lineTo(x + width, y + height - radius);
      ctx.quadraticCurveTo(x + width, y + height, x + width - radius, y + height);
      ctx.lineTo(x + radius, y + height);
      ctx.quadraticCurveTo(x, y + height, x, y + height - radius);
      ctx.lineTo(x, y + radius);
      ctx.quadraticCurveTo(x, y, x + radius, y);
      ctx.closePath();
      ctx.stroke();
      ctx.globalAlpha = 0.6;
      ctx.fill();
      ctx.globalAlpha = 1.0;
      ctx.fillStyle = "#000";
      ctx.textAlign = "center";
      ctx.textBaseline = "middle";
      ctx.fillText(label,x + (width / 2),y + (height / 2));
   }

   function labelClusters(ctx,h,w) {
      if(!("both" == option.autolabel || "cluster" == option.autolabel)) return;

      for(var cluster in clusterLocations) {
         var loc = clusterLocations[cluster];
         var x = Math.floor(loc[0] * w) + 0.5;
         var y = Math.floor(loc[1] * h) + 0.5;
         drawLabel(ctx,x,y,myData.groups[cluster]);
      }
     
   }

   function labelNode(ctx,h,w,node) {
      var loc = nodeLocations[node];
      if(!loc) return;

      var x = loc[0] * w;
      var y = loc[1] * h;
      var label = myData.nodeNames[node];
      if(!label) label = node;
      drawLabel(ctx,x,y,label);
   }

   function labelFlows(ctx,h,w) {
      if(!("both" == option.autolabel || "flow" == option.autolabel)) return;

      var flows = myData.flows;
      if(!flows) return;

      var maxNodeValue = 0;
      var maxNode = null;
      var maxNodeEdges = 0;
      var maxEdgesNode = null;
      var maxEdgeVal = 0;
      var maxEdgeStart = null;
      var maxEdgeEnd = null;

      var nodeVolume = {};
      var nodeEdges = {};
      var edges = {};

      // collect all connections between each node pair
      var edges = {};
      for(var i = 0; i < flows.length; i++) {
         var flow = flows[i];
         var sourcenode = flow[1];
         var destinationnode = flow[3];
         var protocol = flow[4];
         var val = flow[5];
         
         var key;
         if(sourcenode < destinationnode) key = sourcenode + "," + destinationnode;
         else key = destinationnode + "," + sourcenode;

         if(!edges[key]) edges[key] = val;
         else edges[key] += val;
      } 

      for(var edge in edges) {
         var val = edges[edge];
         var parts = edge.split(",");
         if(val > maxEdgeVal) {
           maxEdgeVal = val;
           maxEdgeStart = parts[0];
           maxEdgeEnd   = parts[1];
         }
         if(!nodeVolume[parts[0]]) nodeVolume[parts[0]] = val;
         else nodeVolume[parts[0]] += val;
         if(!nodeVolume[parts[1]]) nodeVolume[parts[1]] = val;
         else nodeVolume[parts[1]] += val;

         if(!nodeEdges[parts[0]]) nodeEdges[parts[0]] = 1;
         else nodeEdges[parts[0]]++;
         if(!nodeEdges[parts[1]]) nodeEdges[parts[1]] = 1;
         else nodeEdges[parts[1]]++;
      }
      
      for(var node in nodeVolume) {
         var val = nodeVolume[node];
         if(val > maxNodeValue) {
           maxNodeValue = val;
           maxNode = node;
         }
      } 

      for(var node in nodeEdges) {
         var val = nodeEdges[node];
         if(val > maxNodeEdges) {
            maxNodeEdges = val;
            maxEdgesNode = node;
         }
      } 

      var nodesToLabel = {};
      if(maxNode) nodesToLabel[maxNode] = maxNode;
      if(maxEdgesNode) nodesToLabel[maxEdgesNode] = maxEdgesNode;
      if(maxEdgeStart) nodesToLabel[maxEdgeStart] = maxEdgeStart;
      if(maxEdgeEnd) nodesToLabel[maxEdgeEnd] = maxEdgeEnd;
      for(var node in nodesToLabel) labelNode(ctx,h,w,node); 
   }

   function drawCircles(ctx,h,w) {
      drawFlows(ctx,h,w);
      drawNodes(ctx,h,w);
      drawClusters(ctx,h,w);

      labelClusters(ctx,h,w);
      labelFlows(ctx,h,w);
   }

   function updateLegend(items) { 
     myTable.find("tbody tr").each(function(row) {
       $(this).find("td").each(function(col) {
         var idx = row * cellsPerRow + Math.floor(col / 2);
         var item = items[idx];
         if(item) {
           if(col % 2 == 0) {
             $(this).find("img").css({"background-color": colors[idx % colors.length], "border-color": ""});
             var val = item[0];
             $(this).find("a").data("evt", {evtId: "PROTOCOL", evtValue: val });   
           } else {
             key = item[0];
             label = item[1];

             $(this).find('a').text(label ? label : "").data("evt", {evtId: "PROTOCOL", evtValue: key});
           }
         } else {
           if(col % 2 == 0) {
              $(this).find("img").css({"background-color": "transparent", "border-color": "transparent"});
             $(this).find("a").removeData("evt");
           } else {
             $(this).find('a').text("").removeData("evt");
           }
         }
       });
     });
   }

   function processResponse(response,data) {
      if(!response) return;

      switch(response.action) {
        case "loadpage":
          if(response.url) window.location = response.url;
          break;
        case "dialog":
          if(response.message) createDialog(
              response.message,
              response.selectFlag,
              response.excludeFlag,
              response.searchFlag,
              data);
          break;
        default:
      }
   }

   function createDialog(message, selectFlag, excludeFlag, searchFlag, data) {
     var args = {};
     if(selectFlag || excludeFlag || searchFlag) {
       var buttons = {};
       if(selectFlag) buttons[option.select] = function() {
          $(this).dialog("close");
          $.getJSON(option.dataURL + "&_evtName=select&_evtId=" + escape(data.evtId) + "&_evtValue=" + escape(data.evtValue), function(response) {
            processResponse(response,data);
          });
       };
       if(excludeFlag) buttons[option.exclude] = function() {
          $(this).dialog("close");
          $.getJSON(option.dataURL + "&_evtName=exclude&_evtId=" + escape(data.evtId) + "&_evtValue=" + escape(data.evtValue), function(response) {
            processResponse(response,data);
          });
       };
       if(searchFlag) buttons[option.search] = function() {
          $(this).dialog("close");
          $.getJSON(option.dataURL + "&_evtName=search&_evtId=" + escape(data.evtId) + "&_evtValue=" + escape(data.evtValue), function(response) {
            processResponse(response,data);
          });
       };
       args["buttons"] = buttons;
     }
     $("<div>" + message + "</div>").dialog(args);
   }

   function click(data) {
       $.getJSON(option.dataURL + "&_evtName=nodeClick&_evtId=" + escape(data.evtId) + "&_evtValue=" + escape(data.evtValue), function(response) {
         processResponse(response,data);
       });
   }

   function clickLegend(event) {
     var data = $(this).data("evt");
     if(data) click(data);
   }

   function csvVal(value) {
     return value ? value : "";
   }

   function clickNode(node) {
     var data = {evtId:"NODE", evtValue: node + "," + csvVal(myData.nodeNames[node])};
     click(data);
   }

   function clickCluster(cluster) {
     var data = {evtId: "CLUSTER", evtValue: cluster};
     click(data);
   }

   function clickLink(start,end) {
     var data = {evtId: "LINK", evtValue: start + "," + csvVal(myData.nodeNames[start]) + "," + end + "," + csvVal(myData.nodeNames[end])};
     click(data);
   }

   function select(x,y) {
      if(!myData) return;
 
      var h = myCanvas.height();
      var w = myCanvas.width(); 

      // first check nodes
      var nt = nodeBoxWidth / 2;
      for(var node in nodeLocations) {
        var loc = nodeLocations[node];
        var nx = loc[0] * w;
        var ny = loc[1] * h;
        var dist = pointDist(x,y,nx,ny);
        if(dist < nt) return clickNode(node);
      }

      // next check clusters
      var gt = clusterBoxWidth / 2;
      for(var cluster in clusterLocations) {
         var loc = clusterLocations[cluster];
         var nx = loc[0] * w;
         var ny = loc[1] * h;
         var dist = pointDist(x,y,nx,ny);
         if(dist < gt) return clickCluster(myData.groups[cluster]);
      }

      // finally flows
      var flows = myData.flows;
      var ft = 2;
      for(var i = 0; i < flows.length; i++) {
         var flow = flows[i];
         var src_node = flow[1];
         var dst_node = flow[3];
         var src_loc = nodeLocations[src_node];
         var dst_loc = nodeLocations[dst_node];
         var src_x = src_loc[0] * w;
         var src_y = src_loc[1] * h;
         var dst_x = dst_loc[0] * w;
         var dst_y = dst_loc[1] * h;
         var dist = pointDistLine(x, y, src_x, src_y, dst_x, dst_y);
         if(dist < ft) return clickLink(src_node,dst_node);
      }
   }

   function initChart() {
      colors = option.colors.split(',');
      var nCells = 1 + parseInt(option.nProtocols);
      var nRows = Math.ceil(nCells / cellsPerRow);
      var table = '<table class="constrain" id="' + myID + '-legend">'; 
      table += "<tbody>";
      for(var r = 0; r < nRows; r++) {
        table += '<tr>';
        for(var c = 0; c < cellsPerRow; c++) {
          table += '<td><a href="javascript:void(0)"><img src="images/enabled.gif" height="10" width="10" border="1" /></a></td>';
          table += '<td class="alignl"><a href="javascript:void(0)"></a></td>';
        }
        table += '</tr>';
      }
      table += "</tbody>";
      table += "</table>";
      myDiv.empty();
      myDiv.append(table);
      myTable = $("#" + myID + "-legend");

      // hookup click and hover functions
      myTable.find("a").click(clickLegend);


      //if(typeof(G_vmlCanvasManager) != 'undefined') {
      //   var canvas = document.createElement('canvas');
      //   canvas.id= myID + '-chart';
      //   canvas.height= option.height;
      //   myDiv.append(canvas);
      //   G_vmlCanvasManager.initElement(canvas);
      //} else {
      myDiv.append('<canvas id="' + myID + '-chart"></canvas>');
      //}

      myCanvas = $("#" + myID + "-chart");
      myCanvas.css("width", "100%");
      myCanvas.css("height", "" + option.height + "px");        

      myCanvas
       .click(function(e) {
         var position = $(this).position();
         var x = e.clientX - position.left;
         var y = e.clientY - position.top;
         select(x,y);
       })
       .hover(function(e) {
         $(this).addClass('hover-bar');
        }, function(e) {
         $(this).removeClass('hover-bar');
       });
      $(window).resize(drawChart);
   };

   return this;
  }
})(jQuery);
