// Copyright (c) InMon Corp. ALL RIGHTS RESERVED

(function($) {
  $.fn.topN = function(settings) {
    
    var defaults = {
        height         : 200,
        bottomInset    : 20,
        leftInset      : 60,
        topInset       : 10,
        rightInset     : 20,
        barSpace       : 2,
        refresh        : false,
        integer        : false,
        select         : "Select",
        exclude        : "Exclude", 
        search         : "Search",
        colors         : "red,orange,yellow,green,blue,violet",
        darkColors     : null,
        valueAxisLabel : null,
        bidirectional  : false,
        dataTimeout    : 600000
    };
    var option = $.extend(defaults,settings);

    var myID = $(this).attr('id');
    var myDiv = $(this);
    var myTable;
    var myCanvas;
    var myData;
    var selectedTime;
    var nkeys;
    var colors;
    var darkColors;
    var yZero;
    var vscale;

    if(option.dataURL) {
         getData();
    }

   function mergeData(update) {
     if(myData) {
        var i = myData.data.length;
        while(myData.data[--i][0] > update.data[0][0]) {}
        var newdata = myData.data.slice(0,i).concat(update.data);
        if(option.intervals && newdata.length > option.intervals) {
           newdata.splice(0,newdata.length - option.intervals);
           myData.data = newdata;
        }
     }
     else myData = update;
   }

   function getData() {
     $.ajax({
        url: option.dataURL + (myData ? "&refresh=true" : ""), 
        success: function(data) {
          mergeData(data);
          drawChart();
          if(option.refresh) setTimeout(getData, 60000);    
        },
        error: function(result,status,errorThrown) {
             myDiv.empty();
             myDiv.append('<span class="warn">' + status + (errorThrown ? '&nbsp;/&nbsp;' + errorThrown : '') + '</span>');
        },
        timeout: option.dataTimeout
      });
   };

   function drawChart() {
      if(!myTable) initChart(); 

      var canvas = myCanvas[0];
      if(!(canvas && canvas.getContext)) return;

      var ctx = canvas.getContext('2d');
      ctx.font = '10px sans-serif';

      var h = myCanvas.height();
      var w = myCanvas.width();

      var ratio = window.devicePixelRatio;
      if(ratio && ratio > 1) {
           canvas.height = h * ratio;
           canvas.width = w * ratio;
           ctx.scale(ratio,ratio);
       }
       else { 
          canvas.height = h;
          canvas.width = w;
      }

      // Draw the x and y axes
      var maxVal = maximumValue(myData['data']);
      var maxUp = maxVal[0];
      var maxDown = maxVal[1]; 

      drawValueAxis(ctx,h,w,maxUp,maxDown); 
      drawTimeAxis(ctx,h,w,maxUp,maxDown);	
   }
  
   var stepsizes = [1,2,5,10,20,50,100,200,500];
   var suffixes = ["\uD835\uDF07","\uD835\uDC5A",,"K","M","G","T","P","E"];
   function drawValueAxis(ctx,h,w,maxUp,maxDown) {
      // find range
      var i = 0;
      var divisor = 0.000001;

      maxVal = maxUp + maxDown;
      for(i = 0; i < suffixes.length; i++) {
         if((maxVal / divisor) < 1000) break;
         divisor *= 1000;
      }
      var steps = Math.floor(maxVal/divisor);
      if(steps < 5 && i > 0) {
         divisor /= 1000;
         i--;
         steps = Math.floor(maxVal / divisor);
      }

      var rOffset = 5;
      ctx.textAlign = 'right';
      ctx.textBaseline = 'middle';
      
      var stepsize = 1;
      var tsteps;
      for(var j = 0; j < stepsizes.length; j++) {
         stepsize = stepsizes[j];
         tsteps = steps / stepsize; 
         if(Math.floor(tsteps) < 8) break;
      }
      steps = tsteps;

      ctx.fillStyle = "#000";

      yZero = Math.floor((h - option.bottomInset - option.topInset) * (maxUp / maxVal)) + option.topInset;
      ctx.fillText("0",option.leftInset - rOffset, yZero);

      vscale = (h - option.bottomInset - option.topInset) / maxVal;
      var vstep = vscale * stepsize * divisor;

      var meas = ctx.measureText("0");
      var maxLabel = meas.width;

      // upAxis
      var yval = yZero - vstep;
      var s = 1;
      while(yval > option.topInset) {
         var lval = s * stepsize;
         var suffix = suffixes[i];
         if(lval >= 1000 && i < (suffixes.length - 1)) {
            lval /= 1000;
            suffix = suffixes[i + 1]; 
         }
         var label = lval + (s && suffix ? suffix : "");
         meas = ctx.measureText(label);
         maxLabel = Math.max(maxLabel,meas.width);
         ctx.fillText(label, option.leftInset - rOffset, yval);
         drawLine(ctx, option.leftInset, yval, w - option.rightInset, yval, "#ccc");
         s++;
         yval -= vstep;
      }

      // downAxis
      yval = yZero + vstep;      
      s = 1; 
      var bottom = h - option.bottomInset;
      while(yval < bottom) {
         var lval = s * stepsize;
         var suffix = suffixes[i];
         if(lval >= 1000 && i < (suffixes.length - 1)) {
            lval /= 1000;
            suffix = suffixes[i + 1];
         }
         var label = lval + (s && suffix ? suffix : "");
         meas = ctx.measureText(label);
         maxLabel = Math.max(maxLabel,meas.width);
         ctx.fillText(label, option.leftInset - rOffset, yval);
         drawLine(ctx, option.leftInset, yval, w - option.rightInset, yval, "#ccc");
         s++;
         yval += vstep;
      }

      // add units
      if(option.valueAxisLabel != null) {
        var font = ctx.font;
        var fontArgs = font.split(' ');
        ctx.font = '12px ' + fontArgs[fontArgs.length - 1];
        ctx.rotate(-Math.PI/2);
        ctx.textAlign = "center";
        ctx.textBaseline = "bottom";
        ctx.fillText(option.valueAxisLabel, -option.topInset - ((h - option.topInset - option.bottomInset) / 2), Math.max(10,option.leftInset - maxLabel - rOffset - 5));
        ctx.rotate(Math.PI/2);
        ctx.font = font;
      }
      drawLine(ctx, option.leftInset, option.topInset, option.leftInset, h - option.bottomInset, "#000");
   }

   function printValue(val) {
      if(val == 0) return "0";

      var i = 0;
      var divisor = 0.000001;

      for(i = 0; i < suffixes.length; i++) {
         if((val /divisor) < 1000) break;
         divisor *= 1000;
      }
      var scaled = val / divisor;
      var fp = suffixes[i] || !option.integer ? 2 : 0;
      return scaled.toFixed(fp) + (suffixes[i] ? suffixes[i] : "");
   }

   function drawTimeAxis(ctx,h,w,maxUp,maxDown) {
     var maxVal = maxUp + maxDown;

     var data = myData["data"];
     var selectedIdx = selectedBar();
     var selectedItems = data[selectedIdx][2];

     updateLegend(selectedItems);
   
     var step = (w - option.leftInset - option.rightInset) / data.length;
     var barWidth = step - option.barSpace;
     var bottom = h - option.bottomInset;
   
     var tOffset = 5;
     var height = h - bottom - tOffset - 2; 
     ctx.textAlign = 'center';
     ctx.textBaseline = 'middle';
     var tMiddle = bottom + tOffset + height / 2;

     var markerPos;
     function countTimeLabels(step) {
        var count = 0;
        for(var t = 0; t < data.length; t++) {
           if(data[t][0] % step == 0) count++;
        }
        return count;
     }
     var tsteps = [1,2,4,6,12,18];
     var tstep;
     for(var i = 0; i < tsteps.length; i++) {
        tstep = 300000 * tsteps[i];
        if(countTimeLabels(tstep) < 20) break;
     } 
     for(var t = 0; t < data.length; t++) {
        // draw bars

        // center of bar
        var xPos = option.leftInset + (t * step) + (step / 2);

        // draw full bar
        var items = data[t][2];
        var sumUp = 0;
        var sumDown = 0;
        for(i = 0; i < items.length; i++) {
           var row = items[i];
           if(option.bidirectional) {
             sumUp += row[row.length - 3];
             sumDown += row[row.length - 2];
           }
           else sumUp += row[row.length - 1];
        }
        ctx.fillStyle="#ccc";

        // up
        var barHeight = sumUp * vscale;
        ctx.fillRect(xPos - (barWidth / 2),yZero -
barHeight,barWidth,barHeight);

        if(option.bidirectional) {
           barHeight = sumDown * vscale;
           ctx.fillRect(xPos - (barWidth / 2), yZero, barWidth, barHeight);
        }

        // now overlay any data from selected bar
        function keysMatch(r1,r2) {
          for(var c = 0; c < nkeys; c++) {
            if(!r1[c] || !r2[c] || r1[c][0] != r2[c][0]) return false;
          }
          return true;
        }
        var elBottomU = yZero;
        var elBottomD = yZero;
        for(var i = 0; i < selectedItems.length; i++) {
           var key = selectedItems[i][0];
           if(key) {
              for(var j = 0; j < items.length; j++) {
                var row = items[j];
                if(keysMatch(selectedItems[i],row)) {
                   var row = items[j];
                   if(option.bidirectional) {
                      var valUp = row[row.length - 3];
                      var hVal = valUp * vscale;
                      ctx.fillStyle = colors[i % colors.length];
                      ctx.fillRect(xPos - (barWidth / 2), elBottomU - hVal, barWidth, hVal);
                      elBottomU -= hVal;

                      var valDown = row[row.length - 2];
                      hVal = valDown * vscale;
                      elBottomD += hVal;
                      ctx.fillStyle = darkColors[i % darkColors.length];
                      ctx.fillRect(xPos - (barWidth / 2), elBottomD - hVal, barWidth, hVal);
                   }
                   else {
                      var val = row[row.length - 1];
                      var hVal = val * vscale;
                      ctx.fillStyle = colors[i % colors.length];
                      ctx.fillRect(xPos - (barWidth / 2), elBottomU -hVal, barWidth, hVal);
                      elBottomU -= hVal;
                  }
                }
              }
           }
        }

        // time labels and tick marks
        var time = data[t][0];
        if(selectedIdx == t) {
           markerPos = xPos;
        } else if(t > 0 && time % tstep == 0) {
           ctx.fillStyle = "#000";
           drawLine(ctx, xPos, bottom, xPos, bottom + 4, "#000");
           ctx.fillText(data[t][1], xPos, tMiddle);
        }

     }
     drawLine(ctx,option.leftInset,yZero,w - option.rightInset,yZero,"#000");
     if(option.bidirectional) drawLine(ctx, option.leftInset, bottom, w - option.rightInset, h - option.bottomInset,"#ccc");
     if(markerPos) {
       ctx.fillStyle = "#000";
       drawLine(ctx, markerPos, bottom, markerPos, bottom + 4, "#000");
       var label = data[selectedIdx][1];
       var meas = ctx.measureText(label);
       ctx.strokeStyle = "#000";
       ctx.fillStyle = "#fff";
       var radius = 2;
       var width = meas.width + 4;
       var x = markerPos - width / 2;
       var y = bottom + tOffset;
       ctx.beginPath();
       ctx.moveTo(x + radius, y);
       ctx.lineTo(x + width - radius, y);
       ctx.quadraticCurveTo(x + width, y, x + width, y + radius);
       ctx.lineTo(x + width, y + height - radius);
       ctx.quadraticCurveTo(x + width, y + height, x + width - radius, y + height);
       ctx.lineTo(x + radius, y + height);
       ctx.quadraticCurveTo(x, y + height, x, y + height - radius);
       ctx.lineTo(x, y + radius);
       ctx.quadraticCurveTo(x, y, x + radius, y);
       ctx.closePath();
       ctx.stroke();
       ctx.fill();
       ctx.fillStyle = "#000";
       ctx.fillText(label,markerPos,tMiddle);
     }
   }

   function drawLine(ctx, startx, starty, endx, endy, style) {
     ctx.beginPath();
     ctx.moveTo(Math.floor(startx) + 0.5, Math.floor(starty) + 0.5);
     ctx.lineTo(Math.floor(endx) + 0.5, Math.floor(endy) + 0.5); ctx.strokeStyle = style;
     ctx.lineWidth = 1;
     ctx.stroke();
   }

   function maximumValue(data) {
     var maxUp = 0;
     var maxDown = 0;
     for(var t = 0; t < data.length; t++) {
       var sumUp = 0;
       var sumDown = 0;
       var vals = data[t][2];
       for(var n = 0; n < vals.length; n++) {
         var row = vals[n];
         if(option.bidirectional) {
            sumUp += row[row.length - 3];
            sumDown += row[row.length - 2];
         }
         else sumUp += row[row.length - 1];
       }
       if(sumUp > maxUp) maxUp = sumUp;
       if(sumDown > maxDown) maxDown = sumDown;
     }
     return [maxUp,maxDown];
   }

   function updateLegend(items) { 
     myTable.find("tbody tr").each(function(row) {
       $(this).find("td").each(function(col) {
         var item = items[row];
         if(item && item[0]) {
           if(col == 0) {
             $(this).find("img").css({"background-color": colors[row % colors.length], "border-color": ""});
             var val = item[0][0];
             for(var i = 1; i < nkeys; i ++) val += "," + item[i][0];
             $(this).find("a").data("evt", {evtId: col, evtValue: val });   
           } else {
             var val = item[col - 1];
             if(col <= nkeys) {
                var key = null;
                var label = null;
                if(val) {
                  key = val[0];
                  if(val.length == 2) label = val[1];
                  else label = key;
                }

                $(this).find('a').text(label ? label : "").data("evt", {evtId: col, evtValue: key});
             } else $(this).text(printValue(val));
           }
         } else {
           if(col == 0) {
             $(this).find("img").css({"background-color": "transparent", "border-color": "transparent"});
             $(this).find("a").removeData("evt");
           } else {
             if(col <= nkeys) $(this).find('a').text("").removeData("evt");
             else $(this).text("");
           }
         }
       });
     });
   }

   function selectedBar() {
     var data = myData['data'];
     for(var i = 0; i < data.length; i++) {
       if(data[i][0] == selectedTime) return i;
     }
     return data.length - 1;
   }

   function selectBar(x,y) {
     if(!myData) return;

     var h = myCanvas.height();
     var w = myCanvas.width();

     var numBars = myData['data'].length;
     if(x > option.leftInset) {
        var idx = Math.floor((x - option.leftInset) * numBars / (w - option.leftInset - option.rightInset));
        selectedTime = idx != numBars - 1 ? myData["data"][idx][0] : null;
        drawChart();
     }
   }

   function processResponse(response,data) {
      if(!response) return;

      switch(response.action) {
        case "loadpage":
          if(response.url) window.location = response.url;
          break;
        case "dialog":
          if(response.message) createDialog(
            response.message,
            response.selectFlag,
            response.excludeFlag,
            response.searchFlag,
            data);
          break;
        default:
      }
   }

   function createDialog(message, selectFlag, excludeFlag, searchFlag, data) {
     var args = {};
     if(selectFlag || excludeFlag || searchFlag) {
       var buttons = {};
       if(selectFlag) buttons[option.select] = function() {
          $(this).dialog("close");
          $.getJSON(option.dataURL + "&_evtName=select&_evtId=" + escape(data.evtId) + "&_evtValue=" + escape(data.evtValue), function(response) {
            processResponse(response,data);
          });
       };
       if(excludeFlag) buttons[option.exclude] = function() {
          $(this).dialog("close");
          $.getJSON(option.dataURL + "&_evtName=exclude&_evtId=" + escape(data.evtId) + "&_evtValue=" + escape(data.evtValue), function(response) {
            processResponse(response,data);
          });
       };
       if(searchFlag) buttons[option.search] = function() {
          $(this).dialog("close");
          $.getJSON(option.dataURL + "&_evtName=search&_evtId=" + escape(data.evtId) + "&_evtValue=" + escape(data.evtValue), function(response) {
            processResponse(response,data);
          });
       };
       args["buttons"] = buttons;
     }
     $("<div>" + message + "</div>").dialog(args);
   }

   function clickAction(event) {
     var data = $(this).data("evt");
     if(data) {
       $.getJSON(option.dataURL + "&_evtName=nodeClick&_evtId=" + escape(data.evtId) + "&_evtValue=" + escape(data.evtValue), function(response) {
         processResponse(response,data);
       });
     }
   }

   function initChart() {
      var labels = option.cnames.split(',');
      colors = option.colors.split(',');
      if(option.darkColors != null) darkColors = option.darkColors.split(',');
      else darkColors = colors;
      var truncate = parseInt(option.truncate);
      var nvals =  option.bidirectional ? 3 : 1;
      nkeys = labels.length - nvals;

      var table = '<table class="constrain" id="' + myID + '-legend">'; 
      table += "<thead>";
      table += "<th></th>";
      for(var i = 0; i < labels.length; i++) table += "<th>" + labels[i] + "</th>";
      table += "</thead>";
      table += "<tbody>";
      for(var i = 0; i < truncate; i++) {
        table += '<tr class="' + (i % 2 == 0 ? 'even' : 'odd') + '">';
        table += '<td><a href="javascript:void(0)"><img src="images/enabled.gif" height="10" width="10" border="1" /></a></td>';
        for(var j = 0; j < nkeys; j++) table += '<td class="alignl"><a href="javascript:void(0)"></a></td>';
        for(var j = 0; j < nvals; j++) table += '<td class="alignr"></td>';
        table += '</tr>';
      }
      table += "</tbody>";
      table += "</table>";
      myDiv.empty();
      myDiv.append(table);
      myTable = $("#" + myID + "-legend");

      // hookup click and hover functions
      myTable.find("a").click(clickAction);


      //if(typeof(G_vmlCanvasManager) != 'undefined') {
      //   var canvas = document.createElement('canvas');
      //   canvas.id= myID + '-chart';
      //   canvas.height= option.height;
      //   myDiv.append(canvas);
      //   G_vmlCanvasManager.initElement(canvas);
      //} else {
      myDiv.append('<canvas id="' + myID + '-chart"></canvas>');
      //}

      myCanvas = $("#" + myID + "-chart");
      myCanvas.css("width", "100%");
      myCanvas.css("height", "" + option.height + "px");        

      myCanvas
       .click(function(e) {
         var position = $(this).position();
         var x = e.clientX - position.left;
         var y = e.clientY - position.top;
         selectBar(x,y);
       })
       .hover(function(e) {
         $(this).addClass('hover-bar');
        }, function(e) {
         $(this).removeClass('hover-bar');
       });
      $(window).resize(drawChart);
   };

   return this;
  }
})(jQuery);
