#!/bin/bash

if [ $# != 1 ]; then
  echo "Usage: $0 {agent}"
  exit 1;
fi
AGENT=$1

# get the path to this script, and use it to find gawk
MYDIR=`dirname $0`;
GAWK=awk
STATEDIR="$MYDIR/../../state/agents";
AGSTATE="$STATEDIR/$AGENT";

$GAWK -vdir="$AGSTATE" -- '

function deleteNonUnique(all, scope, mapping, descr,    revMap, dups) {
    for(idx in all) {
        if(idx in mapping && idx in scope) {
	    revKey = mapping[idx] "." scope[idx];
	    prevIdx = revMap[revKey];
	    if(prevIdx) {
		# print "found dups " idx " and "  prevIdx " in " descr "=" mapping[idx] " for agent " scope[idx];
		dups[prevIdx] = 1;
                dups[idx] = 1;
	    }
	    else {
		revMap[revKey] = idx;
	    }
	}
    }
    # prune out the dups
    for(idx in dups) {
	delete mapping[idx];
    }
    # do not really have to do this because they are declared
    # as local vars above (by being extra function params)
    # but it does not hurt to make sure they are cleared out.
    delete revMap;
    delete dups;
}

BEGIN {
    FS=",";
    OFS=",";
    # inputs
    f_system = dir "/system.txt";
    f_interfaces = dir "/poll.txt";
    f_bridgeport = dir "/bridgePort.txt";
    f_lldp = dir "/lldp.txt";
    f_cdp = dir "/cdp.txt";
    f_fdp = dir "/fdp.txt";
    f_lldploc = dir "/lldploc.txt";
    f_lldpman = dir "/lldpman.txt";
    f_lldpxdot3 = dir "/lldpxdot3.txt";
    f_lagport = dir "/lagPorts.txt";
    f_stack = dir "/stack.txt";
    # outputs
    f_links = dir "/links.txt";
    f_lags = dir "/lags.txt";
    # truncate outputs
    printf "">f_links;
    printf "">f_lags;
    # troubleshooting
    debug = 0;

    # load sysObjectID and extract enterprise - in
    # case we need to adopt a model/vendor-specific
    # approach to any of these steps below.
    while(getline <f_system == 1) {
	if(NF >= 2) {
	    if($1 == "sysObjectID") {
		sysObjectID = $2;
		split(sysObjectID, oid, ".");
		enterprise = oid[7];
	    }
	}
    }
    # load interfaces
    while(getline <f_interfaces == 1) {
	if(NF >= 18) {
	    ifDescr[$13]=$3;
	    ifName[$14]=$3;
	    ifAlias[$15]=$3;
            ifIndices[$3] = $14;
	}
    }
    # load bridgePort -> ifIndex
    while(getline <f_bridgeport == 1) {
	if(NF >= 2) {
	    bridgePort[$1] = $2;
	}
    }
    # construct lldpPortNum -> ifIndex
    while(getline <f_lldploc == 1) {
	if(NF >= 4) {
            mtch="none";
	    if($2 in ifDescr)        { lldpPort[$1] = ifDescr[$2]; mtch="ifDescr";  }
	    else if($4 in ifName)    { lldpPort[$1] = ifName[$4];  mtch="ifName4";  }
	    else if($4 in ifAlias)   { lldpPort[$1] = ifAlias[$4]; mtch="ifAlias4"; }
            else if($1 in ifIndices) { lldpPort[$1] = $1;          mtch="ifIndex";  }
	    else if($2 in ifName)    { lldpPort[$1] = ifName[$2];  mtch="ifName2";  }
	    else if($2 in ifAlias)   { lldpPort[$1] = ifAlias[$2]; mtch="ifAlias2"; }
	    if(debug > 0) {
		print "lldpport for " $1 " is " lldpPort[$1] " -- matched on " mtch;
	    }
	}
    }
    # construct ifIndex -> lldpRemManAddr
    while(getline <f_lldpman == 1) {
	if(NF >= 3) {
	    if($1 in lldpport) {
		ifIndex = lldpPort[$1];
		lldpRemManAddr[ifIndex] = $3;
		linked[ifIndex] = 1;
	    }
	}
    }
    # FDP
    while(getline <f_fdp == 1) {
	if(NF >= 4) {
	    fdpRemoteAddress[$1] = $2;
	    fdpRemoteIfName[$1] = $4;
	    linked[$1] = 1;
	}
    }
    # CDP
    while(getline <f_cdp == 1) {
	if(NF >= 5) {
	    cdpRemoteAddress[$1] = $2;
	    cdpRemoteIfName[$1] = $4;
	    cdpRemoteDeviceId[$1] = $5;
	    linked[$1] = 1;
	}
    }
    # LLDP
    while(getline <f_lldp == 1) {
	if(NF >= 11) {
	    ifIndex = ($11 in lldpPort) ? lldpPort[$11] : $1;
	    lldpRemoteIfDescr[ifIndex] = $4;
	    lldpRemoteSysName[ifIndex] = $5;
	    lldpRemoteChassisId[ifIndex] = $7;
	    lldpRemotePortId[ifIndex] = $9;
	    linked[ifIndex] = 1;
	}
    }

    # build a lookup from ifIndex to remoteAgent.  Here we
    # assume that the concatenation of all possible remote
    # agent fields will form a unique ID.
    for (ifIndex in linked) {
	remoteAgentId[ifIndex] =			  \
	    fdpRemoteAddress[ifIndex] "."		  \
	    cdpRemoteAddress[ifIndex] "."		  \
	    cdpRemoteDeviceId[ifIndex] "."		  \
	    lldpRemoteSysName[ifIndex] "."		  \
	    lldpRemoteChassisId[ifIndex];
    }

    # next, we test for remoteAgent:remotePort uniqueness. Possibly
    # discarding a remotePort field if it is non-unique for that
    # remoteAgent.
    deleteNonUnique(linked, remoteAgentId, fdpRemoteIfName, "fdpRemoteIfName");
    deleteNonUnique(linked, remoteAgentId, cdpRemoteIfName, "cdpRemoteIfName");
    deleteNonUnique(linked, remoteAgentId, lldpRemoteIfDescr, "lldpRemoteIfDescr");
    deleteNonUnique(linked, remoteAgentId, lldpRemotePortId, "lldpRemotePortId");

    # Now we can write out the link info. The ifIndex, followed by 5 fields
    # that might indentify the remote agent, and then 5 fields that might
    # identify the report interface.
    for (ifIndex in linked) {
        lldpRemPID = lldpRemotePortId[ifIndex];
        extraPID = "";
        if(lldpRemPID ~ /Eth[0-9]/) {
          # if a Cisco port said it was, say, "Eth1/7" then it might actually be "Ethernet1/7"
          # so offer that as another alternative, to be used as a last resort. Cannot filter
          # on enterprise=9 for this because it is the other end of the link that sees the
          # shortened id, which could be any switch.
          extraPID = "Ethernet" substr(lldpRemPID, 4);
        }
	print ifIndex,lldpRemoteSysName[ifIndex],lldpRemoteChassisId[ifIndex],fdpRemoteAddress[ifIndex],cdpRemoteAddress[ifIndex],cdpRemoteDeviceId[ifIndex],lldpRemoteIfDescr[ifIndex],lldpRemPID,fdpRemoteIfName[ifIndex],cdpRemoteIfName[ifIndex],extraPID >>f_links;
    }
    # lag ports -- allow that these might be bridgePorts or ifIndex numbers
    allBridgePorts = 1;
    while(getline <f_lagport == 1) {
	if(NF >= 3) {
	    lagPorts_if[$1] = $2;
	    if(!($1 in bridgePort)) allBridgePorts = 0;
	    else lagPorts_bp[bridgePort[$1]] = $2;
	}
    }
    if(allBridgePorts == 1) {
	for(ifIndex in lagPorts_bp) {
	    lagPorts_lacp[ifIndex] = lagPorts_bp[ifIndex];
	    lagComponent[ifIndex] = 1;
	}
    }
    else {
	for(ifIndex in lagPorts_if) {
	    lagPorts_lacp[ifIndex] = lagPorts_if[ifIndex];
	    lagComponent[ifIndex] = 1;
	}
    }
    # lag ports from lldpxdot3
    while(getline <f_lldpxdot3 == 1) {
	if(NF >= 4) {
	    ifIndex = ($1 in lldpPort) ? lldpPort[$1] : $2;
	    lagPorts_xdot3[ifIndex] = $3;
	    lagComponent[ifIndex] = 1;
	}
    }
    # Could read stackTable here too, but it is not so easy to distinguish
    # physical LAG relationships there, and it is not clear that there are
    # any devices that support ifStackTable but not XDP.
    # now write out lagPorts
    for (ifIndex in lagComponent) {
	print ifIndex,lagPorts_lacp[ifIndex],lagPorts_xdot3[ifIndex] >>f_lags;
    }
}'
