var lang = lang || "en";

var network = Network.current();
var qinfo = new QInfo();
var SEP = "_SEP_";
var OTHER = "-other-";

Date.prototype.toJSON = function() { return formatdate(this,'yyyy/MM/dd HH:mm:ss'); };

function Trend(args) {
    args = args || {};
    var spec = {};

    // query parameters
    spec.view = args.view;
    spec.select = args.select;
    spec.type = args.type;
    spec.where = args.where;
    spec.sort = args.sort;
    spec.truncate = args.truncate;
    spec.includeother = args.includeother;
    spec.globaltruncate = args.globaltruncate;

    // chart parameters
    if(args.hasOwnProperty('title')) spec.title = qinfo.localize(lang,'t.'+args.title,args.title);
    if(args.hasOwnProperty('ylabel')) spec.ylabel = qinfo.localize(lang,'t.'+args.ylabel,
        qinfo.localize(lang,'c.'+args.ylabel,args.ylabel));
    if(args.hasOwnProperty('scale')) spec.scale = args.scale;
    if(args.hasOwnProperty('step')) spec.step = args.step;
    if(args.hasOwnProperty('mirror')) spec.mirror = args.mirror;
    if(args.hasOwnProperty('stack')) spec.stack = args.stack;
    if(args.hasOwnProperty('colors')) spec.colors = args.colors.split(',');
    if(args.hasOwnProperty('legendHeadings')) spec.legendHeadings = args.legendHeadings;
    if(args.hasOwnProperty('legendLinks')) spec.legendLinks = args.legendLinks;
    if(args.hasOwnProperty('hrule')) spec.hrule = args.hrule;

    this.spec = spec;
    this.times = [];
    this.trends = [];
}

Trend.prototype.addRow = function(row,table) {
    var i, c, val, scale, key, time = row[0];
    if(!this.table) this.table = table;
    scale = this.spec.scale;
    if('trend' === this.spec.type) {
	this.times.push(time);
	for(i = 1; i < row.length; i++) {
	    c = i - 1;
	    val = row[i];
            if(scale) val *= scale;
	    if(!this.trends[c]) this.trends[c] = [];
	    this.trends[c].push(val);
	}
    }
    else if("topn" === this.spec.type) {
	if(this.times.length === 0) {
	    this.trends.push({});
	    this.times.push(time);
        }
	else if(this.times[this.times.length - 1].getTime() !== time.getTime()) {
	    this.trends.push({});
	    this.times.push(time);
	}
	// assume value is last column
	val  = row[row.length - 1];
	if(scale) val *= scale;	
	key = row.slice(1,row.length - 1);
        if(key.reduce(function(prev,cur) { return prev || cur }, '')) key = key.join(SEP);
	else key = OTHER;
	this.trends[this.trends.length - 1][key] = val;
    }
}

Trend.prototype._legendHeadings = function() {
    if(this.spec.legendHeadings) return this.spec.legendHeadings;

    var headings = this.spec.select.split(',');
    headings = headings.slice(1,this.spec.type == 'topn' ? headings.length - 1 : headings.length);
    return headings.map(function(h) {
	var fun = /\(([^)]+)\)/.exec(h);
	h = fun ? fun[1] : h;
       return qinfo.localize(lang,"c."+h,h);
    });
}

Trend.prototype._legendLinks = function() {
    if(!this.spec.legendLinks) return null;
    if(!this.table) return null;
    if(!this.table.ctypes) return null;
    var i, link, links = this.table.ctypes.slice(1,this.spec.type=='topn' ? this.table.ctypes.length - 1 : this.table.ctypes.length);
    for(i = 0; i < links.length; i++) {
	switch(links[i]) {
	case 'address': link = 'Go?action=host&host={addr}'; break;
	case 'subnet': link = 'Go?action=host&host={subnet}'; break;
	case 'agent': link = 'Go?action=agent&agent={addr}'; break;
	case 'interface': link = 'Go?action=agent&agent={iface}'; break;
	case 'port': link = 'Go?action=protocol&protocol={port}'; break;
	case 'as': link = 'Go?action=aspath&aspath={aspath}'; break;
	default: link = '';
	}
	links[i] = link;
    }
    return links;
}

Trend.prototype.chart = function(id) {
    var i, metric, metricname, links, chart = {};
    chart.id = id;
    if(this.spec.title) chart.title = this.spec.title;
    chart.spec = {
      type:this.spec.type,
      sep:SEP,
      mirror:this.spec.mirror,
      stack: this.spec.stack || this.spec.mirror,
      step:this.spec.step,
      units:this.spec.ylabel 
    };
    if(this.spec.colors) chart.spec.colors = this.spec.colors;
    links = this._legendLinks();
    if(links) chart.spec.legendLinks = links;
    chart.trend = {};
    chart.trend.times = this.times;
    chart.trend.trends = {};
    if('trend' === this.spec.type) {
      chart.spec.legend = this._legendHeadings(),
      chart.spec.metrics = [];
      for(i = 0; i < this.trends.length; i++) {
	metric = 't'+i;
	chart.spec.metrics.push(metric);
	chart.trend.trends[metric] = this.trends[i];
      }
    } else if('topn' === this.spec.type) {
      chart.other = OTHER;
      chart.spec.legendHeadings = this._legendHeadings();
      metric = 't1';
      chart.spec.metric = metric;
      chart.trend.trends[metric] = this.trends;
    }
    return chart;
}

function Trends(args) {
    args = args || {};
    this.columns = args.hasOwnProperty('columns') ? args.columns : 2;
    this.interval = args.interval || 'last-60min,last';
    this.group = args.group || 1;
    this.refresh = (args.refresh || 1) * 60000;
    this.trends = [];
    this.views = {};
}

Trends.prototype._addTrend = function(trend) {
    if(!trend || !trend.spec || !trend.spec.view) return;
    var view = trend.spec.view;

    var jobs = this.views[view];
    if(!jobs) {
	jobs = [];
	this.views[view] = jobs;
    }
    jobs.push(trend);
    this.trends.push(trend);
}

Trends.prototype.addTrend = function(spec) {
    this._addTrend(new Trend(spec));
}

Trends.prototype.addReportTrend = function(args) {
    args = args || {};
    var chart = qinfo.getReportChart(args.group,args.report,args.section,args.instance,lang);
    var legend = [];
    var metrics = [];
    var series = [];
    for(var i = 0; i < chart.yaxis.series.length; i++) {
	legend.push(chart.yaxis.series[i].name);
	series.push(chart.yaxis.series[i].values);
    }
    var spec = {type:'trend', legendHeadings:legend,metrics:metrics,stack:chart.stack};
    var title = args.title || chart.title;
    if(title) spec.title = title;
    var ylabel = args.ylabel || chart.yaxis.label;
    if(ylabel) spec.ylabel = ylabel;
    var trend = new Trend(spec);
    trend.times = chart.xaxis.data;
    trend.trends = series;
    this.trends.push(trend);
}

Trends.prototype.addInterfaceTrend = function(args) {
    args = args || {};
    if(!args.interface) return;
    network.path = args.interface;
    if(!network.isInterface()) return;
    var spec = qinfo.chartSpec("interface",args.chart);
    if(!spec) return;
    spec = JSON.parse(spec);
    delete spec['sort'];
    spec.step = true;
    spec.mirror = true;
    spec.stack = true;
    var title = args.title;
    if(!title) {
	title = network.displayName();
	if(title) {
	    network.path = network.agent();
	    title = network.displayName() + ">" + title;
            network.path = args.interface;
	} else title = args.interface;
    }
    spec.title = title;
    spec.where = 'path='+network.interfaceId();
    spec.type = 'trend';
    spec.view = 'ifcounters';
    this._addTrend(new Trend(spec));
}

 Trends.prototype.addHostTrend = function(args) {
     args = args || {};
     var view = 'host';
     var spec = qinfo.chartSpec("host",args.chart);
     if(!spec) return;
     spec = JSON.parse(spec);
     delete spec['sort'];
     spec.step = true;
     if(args.where) spec.where = args.where;
     if(args.title) spec.title = args.title;
     else {
	 spec.title = qinfo.localize(lang,"hosts.trend.show."+args.chart,null);
     }
     spec.type = 'trend';
     spec.view = 'host';
     this._addTrend(new Trend(spec));
}

Trends.prototype.addServicesTrend = function(args) {
    args = args || {};
    var spec = qinfo.chartSpec("service",args.chart);
    if(!spec) return;
    spec = JSON.parse(spec);
    delete spec['sort'];
    spec.step = true;
    if(args.where) spec.where = args.where;
    if(args.title) spec.title = args.title;
    else {
	spec.title = qinfo.localize(lang,"services.stats.show."+args.chart,args.chart);
    }
    spec.type = 'trend';
    spec.view = 'host';
    this._addTrend(new Trend(spec));
}

 Trends.prototype.addTopN = function(view,args) {
     args = args || {};
     var spec = {type:'topn', view:view};
     var value = args.value;
     if('traffic' === view) {
	 value = value ||  'bps';
	 // shortcuts for standard TopN charts
	 if(args.hasOwnProperty('chart')) {
	     if(!args.hasOwnProperty('stack')) args.stack = true;
	     switch(args.chart) {
	     case 'sources':
		 if(!args.hasOwnProperty('title')) args.title = 'topSources';
		 if(!args.hasOwnProperty('keys')) args.keys = 'sourceaddress';
		 break;
	     case 'destinations':
		 if(!args.hasOwnProperty('title')) args.title = 'topDestinations';
		 if(!args.hasOwnProperty('keys')) args.keys = 'destinationaddress';
		 break;
	     case 'pairs':
		 if(!args.hasOwnProperty('title')) args.title = 'topPairs';
		 if(!args.hasOwnProperty('keys')) args.keys = 'sourceaddress,destinationaddress';
		 break;
	     case 'protocols':
		 if(!args.hasOwnProperty('protocols')) args.title = 'topProtocols';
		 if(!args.hasOwnProperty('keys')) args.keys = 'serverport';
		 break;
	     }
	 }
	 if(args.title) spec.title = qinfo.localize(lang,'monitor.chart.'+args.title,args.title);
	 switch(value) {
         case 'bps':
           value = 'rate(bytes)';
           spec.scale = 8;
           spec.ylabel = 'bps';
           break;
         case 'Bps':
            value = 'rate(bytes)';
            spec.ylabel = 'Bps';
            break;
         case 'fps':
           value = 'rate(frames)';
           spec.ylabel = 'fps';
           break;
	 }
     }
     else if('application' === view) {
	 value = value || 'ops';
	 // shortcuts for standard TopN charts
	 if(args.hasOwnProperty('chart')) {
	 }
	 if(args.title) spec.title = qinfo.localize(lang,'services.trend.show.'+args.title,args.title);
	 switch(value) {
	 case 'ops':
	     value = 'rate(op_count)';
	     spec.ylabel = 'ops';
	     break;
	 case 'bps':
	     value = 'rate(op_bytes)';
	     spec.scale = 8;
	     spec.ylabel = 'bps';
	     break;
	 case 'Bps':
	     value = 'rate(op_bytes)';
	     spec.ylabel = 'Bps';
	     break;
	 case 'duration':
	     value = 'mean(op_duration)';
	     spec.ylabel = 'duration';
	     spec.scale = 0.000001;
	     break;
	 }
     }
     if(!value) return;

     spec.select = 'time';
     if(args.keys) spec.select = spec.select + ',' + args.keys;
     spec.select = spec.select + ',' + value;
     spec.colors = '#ff0000,#ffc800,#eeee00,#00ff00,#4040ff,#a020f0';
     spec.sort = value;
     spec.step = true;
     spec.truncate = args.truncate || 5;
     spec.stack = args.hasOwnProperty('stack') ? args.stack : true;
     spec.legendLinks = args.hasOwnProperty('legendLinks') ? args.legendLinks : true;
     if(args.scale) spec.scale = args.scale;
     if(args.ylabel) spec.ylabel = args.ylabel;
     if(args.stack) spec.stack = args.stack;
     if(args.where) spec.where = args.where;
     if(args.legendHeadings) spec.legendHeadings = args.legendHeadings;
     this._addTrend(new Trend(spec));
}

Trends.prototype.addTrafficTopN = function(args) {
    this.addTopN('traffic',args);
}

Trends.prototype.addServicesTopN = function(args) {
    this.addTopN('application',args);
}

Trends.prototype.printResult = function() {
    var range, data, view, i, trends, trend, spec, select, filter, sort, q, f, id, charts = [];
    for(view in this.views) {
	q = new Query();
	q.multiquery = true;
	q.view = view;
	q.interval = this.interval;
	q.group = this.group;
	q.select = [];
	q.where = [];
	q.sort = [];
	q.truncate = [];
	q.includeother = [];
	q.globaltruncate = [];
	q.threads = 1;
	f = [];
   
	function getRowFunction(view) {
	    return function(row,table) { view.addRow(row,table); }
	}

	trends = this.views[view];
	for(i = 0; i < trends.length; i++) {
	    trend = trends[i];
	    spec = trend.spec;
	    q.select.push(spec.select);
	    q.where.push(spec.where);
	    q.sort.push(spec.sort);
	    q.truncate.push(spec.truncate); 
	    q.includeother.push(spec.includeother);
	    q.globaltruncate.push(spec.globaltruncate);
	    f.push(getRowFunction(trends[i]));
	}
        range = range || q.timeRange();
	q.run(f);
    }
    for(i = 0; i < this.trends.length; i++) {
	id="c" + i;
	trend = this.trends[i];
	charts.push(trend.chart(id)); 
    } 
    data = {columns:this.columns,charts:charts}; 
    if(range && ((new Date()).getTime() - range[1].getTime()) > 600000) data.refresh = 0;
    else data.refresh = this.refresh;
   
    println(JSON.stringify(data,null,1));
} 

