-----------------------------------------------------------------------
----- Copyright (C) InMon Corp. 2014 ALL RIGHTS RESERVED           ----
----- Module to send measurements into Traffic Sentinel            ----
-----------------------------------------------------------------------

local json = require("dkjson")
local state = require("state")
local measurement = {}

function measurement.accumulate(obj, field, value, aggmode)
   local val = tonumber(value) or 0
   if(not obj[field]) then obj[field] = 0.0 end
   if(aggmode == "sum") then
      obj[field] = obj[field] + val
   elseif (aggmode == "max") then
      if((obj[field] == nil) or (val > obj[field])) then obj[field] = val end
   elseif (aggmode == "min") then
      if((obj[field] == nil) or (val < obj[field])) then obj[field] = val end
   else
      throw("unknown aggmode")
   end
end

function measurement.iftable(ag, ifindex, settings)
   local mm = {}
   mm.agent = ag
   mm.ds_class = 0
   mm.ds_index = tonumber(ifindex)
   mm.ifTable = settings
   mm.ifTable.ifIndex = mm.ds_index
   local mm_json = json.encode(mm, { indent=true })
   inx.measurement(mm_json)
end

function measurement.countersample(ag, dsclass, dsindex, csamplename, csampleObj)
   local mm = {}
   mm.agent = ag
   mm.ds_class = tonumber(dsclass)
   mm.ds_index = tonumber(dsindex)
   local cs = {}
   mm.counter_sample = cs
   cs[csamplename]=csampleObj
   local mm_json = json.encode(mm, { indent=true })
   -- inx.log(0, "sending json countersample: " .. mm_json)
   inx.measurement(mm_json)
end

function measurement.if_counters(ag, ifindex, type, speed, direction, status, inoctets, inucasts, inmcasts, inbcasts, indiscards, inerrors, unknownprotos, outoctets, outucasts, outmcasts, outbcasts, outdiscards, outerrors, promisc)
   local gen = {}
   gen.if_index = tonumber(ifindex)
   gen.if_type = tonumber(type)
   gen.if_speed = tonumber(speed)
   gen.if_direction = tonumber(direction)
   gen.if_status = tonumber(status)
   gen.if_in_octets = tonumber(inoctets)
   gen.if_in_ucasts = tonumber(inucasts)
   gen.if_in_mcasts = tonumber(inmcasts)
   gen.if_in_bcasts = tonumber(inbcasts)
   gen.if_in_discards = tonumber(indiscards)
   gen.if_in_errors = tonumber(inerrors)
   gen.if_in_unknown_protos = tonumber(unknownprotos)
   gen.if_out_octets_ = tonumber(outoctets)
   gen.if_out_ucasts = tonumber(outucasts)
   gen.if_out_mcasts = tonumber(outmcasts)
   gen.if_out_bcasts = tonumber(outbcasts)
   gen.if_out_discards = tonumber(outdiscards)
   gen.if_out_errors = tonumber(outerrors)
   gen.if_promiscuous_mode = tonumber(promisc)
   measurement.countersample(ag, 0, tonumber(ifindex), "if_counters", gen)
end

function measurement.if_counters_xdr(ag, ifindex, type, speed, direction, status, inoctets, inucasts, inmcasts, inbcasts, indiscards, inerrors, unknownprotos, outoctets, outucasts, outmcasts, outbcasts, outdiscards, outerrors, promisc, portname)
  local dsclass = 0
  local dsindex = tonumber(ifindex)
  local ds = state.getDataSource(ag, dsclass, dsindex)
  local xx = xdr.create()
  xdr.enc_int32(xx, 4) -- SFLCOUNTERS_SAMPLE_EXPANDED
  local cs_offset = xdr.bytes(xx)
  xdr.enc_int32(xx, 0) -- placeholder for CS length
  xdr.enc_int32(xx, ds.seqNo)
  ds.seqNo = ds.seqNo + 1
  xdr.enc_int32(xx, dsclass)
  xdr.enc_int32(xx, dsindex)
  xdr.enc_int32(xx, 2) -- two elements (counters and portname)

  xdr.enc_int32(xx, 1) -- SFLCOUNTERS_GENERIC
  local elem_offset = xdr.bytes(xx)
  xdr.enc_int32(xx, 0) -- placeholder for CS-element length

  xdr.enc_int32(xx, tonumber(ifindex))
  xdr.enc_int32(xx, tonumber(type) or 0)
  xdr.enc_int64(xx, tonumber(speed) or 0)
  xdr.enc_int32(xx, tonumber(direction) or 0)
  xdr.enc_int32(xx, tonumber(status) or 3)
  xdr.enc_int64(xx, tonumber(inoctets) or -1)
  xdr.enc_int32(xx, tonumber(inucasts) or -1)
  xdr.enc_int32(xx, tonumber(inmcasts) or -1)
  xdr.enc_int32(xx, tonumber(inbcasts) or -1)
  xdr.enc_int32(xx, tonumber(indiscards) or -1)
  xdr.enc_int32(xx, tonumber(inerrors) or -1)
  xdr.enc_int32(xx, tonumber(unknownprotos) or -1)
  xdr.enc_int64(xx, tonumber(outoctets) or -1)
  xdr.enc_int32(xx, tonumber(outucasts) or -1)
  xdr.enc_int32(xx, tonumber(outmcasts) or -1)
  xdr.enc_int32(xx, tonumber(outbcasts) or -1)
  xdr.enc_int32(xx, tonumber(outdiscards) or -1)
  xdr.enc_int32(xx, tonumber(outerrors) or -1)
  xdr.enc_int32(xx, tonumber(promisc) or 0)
  xdr.enc_int32_at(xx, (xdr.bytes(xx) - elem_offset - 4), elem_offset)

  xdr.enc_int32(xx, 1005) -- SFLCOUNTERS_PORTNAME
  elem_offset = xdr.bytes(xx)
  xdr.enc_int32(xx, 0) -- placeholder for portname element length
  xdr.enc_str(xx, tostring(portname))
  xdr.enc_int32_at(xx, (xdr.bytes(xx) - elem_offset - 4), elem_offset)
  xdr.enc_int32_at(xx, (xdr.bytes(xx) - cs_offset - 4), cs_offset)
  xdr.send(xx, ag)
  xdr.destroy(xx)
end

function measurement.lag(ag, ifindex, lagifindex, actorMac, partnerMac)
  local dsclass = 0
  local dsindex = tonumber(ifindex)
  local ds = state.getDataSource(ag, dsclass, dsindex)
  local xx = xdr.create()
  xdr.enc_int32(xx, 4) -- SFLCOUNTERS_SAMPLE_EXPANDED
  local cs_offset = xdr.bytes(xx)
  xdr.enc_int32(xx, 0) -- placeholder for CS length
  xdr.enc_int32(xx, ds.seqNo)
  ds.seqNo = ds.seqNo + 1
  xdr.enc_int32(xx, dsclass)
  xdr.enc_int32(xx, dsindex)
  xdr.enc_int32(xx, 1) -- one element (sflow lag counters)

  xdr.enc_int32(xx, 7) -- SFLCOUNTERS_LAG
  local elem_offset = xdr.bytes(xx)
  xdr.enc_int32(xx, 0) -- placeholder for CS-element length

  xdr.enc_mac(xx, actorMac)
  xdr.enc_mac(xx, partnerMac)
  xdr.enc_int32(xx, tonumber(lagifindex) or 0)
  xdr.enc_int32(xx, 0) -- state
  xdr.enc_int32(xx, -1) -- rx
  xdr.enc_int32(xx, -1) -- rx marker
  xdr.enc_int32(xx, -1) -- rx resp
  xdr.enc_int32(xx, -1) -- rx unknown
  xdr.enc_int32(xx, -1) -- rx illegal
  xdr.enc_int32(xx, -1) -- tx
  xdr.enc_int32(xx, -1) -- tx marker
  xdr.enc_int32(xx, -1) -- tx resp
  xdr.enc_int32_at(xx, (xdr.bytes(xx) - elem_offset - 4), elem_offset)
  xdr.enc_int32_at(xx, (xdr.bytes(xx) - cs_offset - 4), cs_offset)
  xdr.send(xx, ag)
  xdr.destroy(xx)
end

function measurement.processor(ag, mtotal, mused, cpu60)
   local tot = tonumber(mtotal) or 0
   local usd = tonumber(mused) or 0
   local cpu = tonumber(cpu60) or 0
   local pr = {}
   pr.total_memory = tot
   pr.free_memory = (tot - usd)
   pr.one_min_cpu = cpu
   measurement.countersample(ag, 0, 0x3FFFFFFF, "processor", pr)
end

-- processorMax: variant that is easier to populate for multi-cpu systems
function measurement.processorMax(ag, cpu60, memutil)
   local cpu = tonumber(cpu60) or 0
   local mem = tonumber(memutil) or 0
   local pr = {}
   pr.cpu60_max = cpu
   pr.memutil_max = memutil
   measurement.countersample(ag, 0, 0x3FFFFFFF, "processorMax", pr)
end

function measurement.env(ag, power_w, temp_c, humidity_pc, fan_rpm, fancount, fanfailed, psucount, psufailed)
   local ev = {}
   ev.power = tonumber(power_w) or 0
   ev.temp = tonumber(temp_c) or 0
   ev.humidity = tonumber(humidity_pc) or 0
   ev.fanrpm = tonumber(fan_rpm) or 0
   ev.fans = tonumber(fancount) or 0
   ev.fans_failed = tonumber(fanfailed) or 0
   ev.psus = tonumber(psucount) or 0
   ev.psus_failed = tonumber(psufailed) or 0
   measurement.countersample(ag, 0, 0x3FFFFFFF, "env", ev)
end

function measurement.sfp_lane(idx, tx_uW, rx_uW, tx_uA, nM)
   local lane = {
       lane_index = tonumber(idx) or 0,
       tx_bias_current = tonumber(tx_uA) or 0,
       tx_power = tonumber(tx_uW) or 0,
       rx_power = tonumber(rx_uW) or 0,
       tx_wavelength = tonumber(nM) or 0,
       rx_wavelength = tonumber(nM) or 0
   }
   return lane
end

measurement.rtmetric_types = {
   "string",
   "counter32",		    
   "counter64",		    
   "gauge32",		    
   "gauge64",		    
   "gaugefloat",		    
   "gaugedouble",
}

local function rtmetric_type_tag(mtype)
   for tt,type in ipairs(rtmetric_types) do
      if (type == mtype) then return tt end
   end
   error("unknown rtmetric type: " .. mtype);
   return -1
end

function measurement.rtmetric_json(ag, ds_class, ds_index, mname, mindex, mtype, mvalue)
   local rtm = {}
   rtm.mname = tostring(mname)
   rtm.mindex = tostring(mindex)
   rtm.mtype = tostring(mtype)
   if(mtype == "string") then
      rtm.mvalue = tostring(mvalue)
   else
      rtm.mvalue = tonumber(mvalue) or 0
   end
   -- inx.log(0, "sending rtmetric: " .. json.encode(rtm, { indent=true }) )
   measurement.countersample(ag, ds_class, ds_index, "rtmetric", rtm)
end

function measurement.rtmetric(ag, ds_class, ds_index, mname, mindex, mtype, mvalue)
  local xx = xdr.create()
  local tag_rtmetric = (inx.bitOP(4300, "<", 12) + 1002)
  xdr.enc_int32(xx, tag_rtmetric)

  local rtm_len_offset = xdr.bytes(xx)
  xdr.enc_int32(xx, 0) -- placeholder for rtm_len

  xdr.enc_str(xx, mindex)
  xdr.enc_int32(xx, 1) -- num fields
  xdr.enc_str(xx, mname)

  if(mtype == "string") then
    xdr.enc_int32(xx, 0)
    xdr.enc_str(xx, tostring(mvalue))
  elseif (mtype == "counter32") then
    xdr.enc_int32(xx, 1)
    xdr.enc_int32(xx, tonumber(mvalue))
  elseif (mtype == "counter64") then
    xdr.enc_int32(xx, 2)
    xdr.enc_int64(xx, tonumber(mvalue))
  elseif (mtype == "gauge32") then
    xdr.enc_int32(xx, 3)
    xdr.enc_int32(xx, tonumber(mvalue))
  elseif (mtype == "gauge64") then
    xdr.enc_int32(xx, 4)
    xdr.enc_int64(xx, tonumber(mvalue))
  elseif (mtype == "gaugefloat") then
    xdr.enc_int32(xx, 5)
    xdr.enc_float(xx, tonumber(mvalue))
  elseif (mtype == "gaugedouble") then
    xdr.enc_int32(xx, 6)
    xdr.enc_double(xx, tonumber(mvalue))
  end
  local offset = xdr.bytes(xx)
  xdr.enc_int32_at(xx, (offset - rtm_len_offset - 4), rtm_len_offset)
  xdr.send(xx, ag)
  xdr.destroy(xx)
end

function measurement.sfp(ag, id, mC, mV, num_lanes, lanes)
  local dsclass = 0
  local dsindex = id
  local ds = state.getDataSource(ag, dsclass, dsindex)
  local xx = xdr.create()
  xdr.enc_int32(xx, 4) -- SFLCOUNTERS_SAMPLE_EXPANDED
  local cs_offset = xdr.bytes(xx)
  xdr.enc_int32(xx, 0) -- placeholder for CS length
  xdr.enc_int32(xx, ds.seqNo)
  ds.seqNo = ds.seqNo + 1
  xdr.enc_int32(xx, dsclass)
  xdr.enc_int32(xx, dsindex)
  xdr.enc_int32(xx, 1) -- one element

  xdr.enc_int32(xx, 10) -- SFLCOUNTERS_SFP
  local elem_offset = xdr.bytes(xx)
  xdr.enc_int32(xx, 0) -- placeholder for CS-element length
  xdr.enc_int32(xx, dsindex)
  xdr.enc_int32(xx, 0) -- mod total lanes
  xdr.enc_int32(xx, tonumber(mV) or 0)
  xdr.enc_int32(xx, tonumber(mC) or 0)
  xdr.enc_int32(xx, num_lanes)
  for ll,lane in pairs(lanes) do
    xdr.enc_int32(xx, ll)
    xdr.enc_int32(xx, lane.tx_bias_current)
    xdr.enc_int32(xx, lane.tx_power)
    xdr.enc_int32(xx, 0) -- tx_power_min
    xdr.enc_int32(xx, 0) -- tx_power_max
    xdr.enc_int32(xx, lane.tx_wavelength)
    xdr.enc_int32(xx, lane.rx_power)
    xdr.enc_int32(xx, 0) -- rx_power_min
    xdr.enc_int32(xx, 0) -- rx_power_max
    xdr.enc_int32(xx, lane.rx_wavelength)
  end
  local end_offset = xdr.bytes(xx)
  xdr.enc_int32_at(xx, (end_offset - elem_offset - 4), elem_offset)
  xdr.enc_int32_at(xx, (end_offset - cs_offset - 4), cs_offset)
  xdr.send(xx, ag)
  xdr.destroy(xx)
end


return measurement
