-----------------------------------------------------------------------
----- Copyright (C) InMon Corp. 2014 ALL RIGHTS RESERVED       --------
----- Module to poll 64-bit counters from a Ciena PacketWave   --------
-----------------------------------------------------------------------

local measurement = require("measurement")

local ciena_pattern = "^1%.3%.6%.1%.4%.1%.1271%."

local e_ciena = "1.3.6.1.4.1.1271."
local t_ciena_ctrs = e_ciena .. "2.3.1.2.3.1.5.1."
local o_inoctets = 1
local o_inucasts = 2
local o_inerrors = 3
local o_inmcasts = 4
local o_inbcasts = 5
local o_outoctets = 16
local o_outucasts = 18
local o_outbcasts = 19
local o_outmcasts = 20
local o_outerrors = 24

local t_iftable = "1.3.6.1.2.1.2.2.1."
local o_iftype = 3
local o_ifspeed = 5
local o_admin = 7
local o_oper = 8
local o_indiscards = 13
local o_inerrors = 14
local o_outdiscards = 19
local o_outerrors = 20

local tw_ciena1 = inx.TWCreate({
      t_iftable .. o_iftype,
      t_iftable .. o_ifspeed,
      t_iftable .. o_admin,
      t_iftable .. o_oper,
      t_iftable .. o_indiscards,
      t_iftable .. o_inerrors,
      t_iftable .. o_outdiscards,
      t_iftable .. o_outerrors
}, "pollCounters_Ciena")

local tw_ciena2 = inx.TWCreate({
      t_ciena_ctrs .. o_inoctets,
      t_ciena_ctrs .. o_inucasts,
      t_ciena_ctrs .. o_inmcasts,
      t_ciena_ctrs .. o_inbcasts,
      t_ciena_ctrs .. o_outoctets,
      t_ciena_ctrs .. o_outucasts,
      t_ciena_ctrs .. o_outmcasts,
      t_ciena_ctrs .. o_outbcasts,
}, "pollCounters_Ciena")

local agents = {}

local function getAgent(ip)
   if(not agents[ip]) then agents[ip] = { ip=ip, interfaces={}} end
   return agents[ip]
end

local function getInterface(ip, if_index)
   local agent = getAgent(ip)
   if(not agent.interfaces[if_index]) then agent.interfaces[if_index] = { if_index=if_index } end
   return agent.interfaces[if_index]
end

local function ciena_ifindex(tableindex)
   return tonumber(tableindex) - 13000 + 100000
end

local function oid_column_and_index(oid)
   local last, secondlast
   for ss in string.gmatch(oid, "%d+") do
      secondlast = last
      last = ss
   end
   return tonumber(secondlast), tonumber(last)
end

local function walkCB2(ip, status, rowid, in_octets, in_ucasts, in_mcasts, in_bcasts, out_octets, out_ucasts, out_mcasts, out_bcasts)
   local agent = getAgent(ip)
   if(status == "ABORT") then
      inx.log(0,"Ciena walkCB2 abort: " .. ip)
   elseif(status == "END") then
      if(agent.debug) then
	 inx.log(0,"Ciena walkCB2 normal termination")
      end
   else
      if(agent.debug) then
	 inx.log(0, "Ciena walkCB2: ip=" .. ip .. " rowid=" .. rowid)
      end

      local col,tableindex = oid_column_and_index(rowid)
      local ifindex = ciena_ifindex(tableindex)
      local intf = getInterface(ip, ifindex)
      intf.if_in_octets = tonumber(in_octets)
      intf.if_in_ucasts = tonumber(in_ucasts)
      intf.if_in_mcasts = tonumber(in_mcasts)
      intf.if_in_bcasts = tonumber(in_bcasts)
      intf.if_out_octets = tonumber(out_octets)
      intf.if_out_ucasts = tonumber(out_ucasts)
      intf.if_out_mcasts = tonumber(out_mcasts)
      intf.if_out_bcasts = tonumber(out_bcasts)
      measurement.countersample(agent, 0, intf.if_index, "if_counters", intf)
   end
   return true -- keep going
end

local function walkCB1(ip, status, rowid, ifType, ifSpeed, ifAdmin, ifOper, in_discards, in_errors, out_discards, out_errors)
   local agent = getAgent(ip)
   if(status == "ABORT") then
      inx.log(0,"Ciena walkCB1 abort: " .. ip)
   elseif(status == "END") then
      if(agent.debug) then
	 inx.log(0,"Ciena walkCB1 normal termination, starting walk2 for " .. ip)
      end
      inx.TWDoTableWalk(agent.twc2, walkCB2)
   else
      if(agent.debug) then
	 inx.log(0, "Ciena walkCB1: ip=" .. ip .. " rowid=" .. rowid)
      end
      local col,ifindex = oid_column_and_index(rowid)
      local intf = getInterface(ip, ifindex)
      intf.if_type = tonumber(ifType)
      intf.if_speed = tonumber(ifSpeed)
      local adminUp = inx.bitOP(tonumber(ifAdmin), "&", 1)
      local operUp = inx.bitOP(tonumber(ifOper), "&", 1)
      intf.if_status = inx.bitOP(operUp, "<", 1) + adminUp
      intf.if_in_discards = tonumber(in_discards)
      intf.if_out_discards = tonumber(out_discards)
      intf.if_in_errors = tonumber(in_errors)
      intf.if_out_errors = tonumber(out_errors)
   end
   return true -- keep going
end

local function sendWALK(args)
   if(args.agent.debug) then inx.log(0,"Ciena sendWALK: " .. args.agent.ip) end
   inx.timer(sendWALK, args, args.mS)
   if(args.agent.up) then
      inx.TWDoTableWalk(args.agent.twc1, walkCB1)
   end
end

local function doAgent(ip, objId, up, args)
   if(string.find(objId, ciena_pattern)) then
      local agent = getAgent(ip)
      agent.up = up
      if(ip == args.debugAgent) then
	 inx.log(0,"Ciena doAgent ip=" .. ip .. " objId=" .. tostring(objId) .. " up=" .. tostring(up))
	 agent.debug = true
      end
      if(up and not agent.twc) then
	 agent.twc1 = inx.TWContextCreate(tw_ciena1, ip)
	 agent.twc2 = inx.TWContextCreate(tw_ciena2, ip)
	 local poll_mS = 1000 * args.interval
	 inx.timer(sendWALK, { agent=agent, mS=poll_mS }, math.random(poll_mS))
      end
   end
end

local pollCpuMem_Ciena = { agentCB = doAgent }
return pollCpuMem_Ciena
