-----------------------------------------------------------------------
----- Copyright (C) InMon Corp. 2014 ALL RIGHTS RESERVED           ----
----- Module to poll CPU and memory stats from Brocade(TM) devices ----
-----------------------------------------------------------------------

local measurement = require("measurement")
local logger = require("logger")
local state = require("state")
local mod = "Brocade_CpuMem"

local e_brocade = "1.3.6.1.4.1.1991."
local e_brocade_mem_total = e_brocade .. "1.1.2.1.54"
local e_brocade_mem_free  = e_brocade .. "1.1.2.1.55"
local e_brocade_cpu       = e_brocade .. "1.1.2.11.1.1.4"

local function preWalkCB(ip)
   local agent,st = state.getState(mod, ip)
   st.mem_total = 0
   st.mem_free = 0
   st.cpu60 = 0
end

local function walkCB(ip, status, rowid, mem_total, mem_free, cpu60)
   local agent,st = state.getState(mod, ip)
   if (status == nil) then
      measurement.accumulate(st, "mem_total", mem_total, "sum")
      measurement.accumulate(st, "mem_free", mem_free, "sum")
      measurement.accumulate(st, "cpu60", cpu60, "sum")
   elseif (status == "END") then
      logger.dbg(mod, agent, "walkCB normal termination, send processor sample")
      measurement.processor(ip, st.mem_total, (st.mem_total - st.mem_free), st.cpu60)
   end
   return true -- keep going
end

return {
   mod = mod,
   objIds = { e_brocade },
   oids = { e_brocade_mem_total, e_brocade_mem_free, e_brocade_cpu },
   preWalkCB = preWalkCB,
   walkCB = walkCB
}
