-----------------------------------------------------------------------
----- Copyright (C) InMon Corp. 2014 ALL RIGHTS RESERVED         ------
----- Module to poll CPU and memory stats from Cisco(TM) devices ------
-----------------------------------------------------------------------

local measurement = require("measurement")
local logger = require("logger")
local state = require("state")

local mod = "Cisco_CpuMem"
local e_cisco = "1.3.6.1.4.1.9."
local e_cisco_mem_pool = e_cisco .. "9.48.1.1.1"
local e_cisco_mem_used = e_cisco_mem_pool .. ".5"
local e_cisco_mem_free = e_cisco_mem_pool .. ".6"
local e_cisco_cpu = e_cisco .. "9.109.1.1.1.1.7"

local function preWalkCB(ip)
   local agent,st = state.getState(mod, ip)
   st.mem_used = 0
   st.mem_free = 0
   st.cpu60 = 0
end

local function walkCB(ip, status, rowid, mem_used, mem_free, cpu60)
   local agent,st = state.getState(mod, ip)
   if (status == nil) then
      measurement.accumulate(st, "mem_used", mem_used, "sum")
      measurement.accumulate(st, "mem_free", mem_free, "sum")
      measurement.accumulate(st, "cpu60", cpu60, "max")
   elseif (status == "END") then
      logger.dbg(mod, agent, "Cisco walkCB normal termination, send processor sample")
      measurement.processor(ip, (st.mem_used + st.mem_free), st.mem_used, st.cpu60)
   end
   return true -- keep going
end

return {
   mod = mod,
   objIds = { e_cisco },
   oids = { e_cisco_mem_used, e_cisco_mem_free, e_cisco_cpu },
   preWalkCB = preWalkCB,
   walkCB = walkCB
}
