-----------------------------------------------------------------------
----- Copyright (C) InMon Corp. 2014 ALL RIGHTS RESERVED       --------
----- Module to poll CPU and memory stats from devices         --------
----- that support the HOST-RESOURCES-MIB                      --------
-----------------------------------------------------------------------

local measurement = require("measurement")
local logger = require("logger")
local state = require("state")
local mod = "HR"
local e_arista = "1.3.6.1.4.1.30065."
local e_net_snmp = "1.3.6.1.4.1.8072."

local host_resources = "1.3.6.1.2.1.25"
local hr_storage_type = host_resources .. ".2.3.1.2"
local hr_storage_type_ram = host_resources .. ".2.1.2"
local hr_storage_size = host_resources .. ".2.3.1.5"
local hr_storage_used = host_resources .. ".2.3.1.6"
local hr_mem_total = host_resources .. ".2.2"
local hr_cpu_pc = host_resources .. ".3.3.1.2"

local function preWalkCB(ip)
   local agent,st = state.getState(mod, ip)
   st.mem_total = 0
   st.mem_total2 = 0
   st.mem_used = 0
   st.cpu60 = 0
end

local function walkCB(ip, status, rowid, mem_total, cpu60, storage_type, storage_size, storage_used)
   local agent,st = state.getState(mod, ip)
   if (status == nil) then
      measurement.accumulate(st, "mem_total", mem_total, "sum")
      measurement.accumulate(st, "cpu60", cpu60, "sum")
      if (storage_type and storage_type == hr_storage_type_ram) then
	 measurement.accumulate(st, "mem_total2", storage_size, "sum")
	 measurement.accumulate(st, "mem_used", storage_used, "sum")
      end
   elseif (status == "END") then
      measurement.processor(ip, st.mem_total, st.mem_used, st.cpu60)
   end
   return true -- keep going
end

return {
   mod = mod,
   objIds = { e_arista, e_net_snmp },
   oids = { hr_mem_total, hr_cpu_pc, hr_storage_type, hr_storage_size, hr_storage_used },
   preWalkCB = preWalkCB,
   walkCB = walkCB
}
