-----------------------------------------------------------------------
----- Copyright (C) InMon Corp. 2014 ALL RIGHTS RESERVED           ----
----- Module to poll temp/PSU/fan data from Brocade(TM) devices    ----
-----------------------------------------------------------------------

local measurement = require("measurement")
local logger = require("logger")
local state = require("state")

local mod = "Brocade_Env"

local e_brocade = "1.3.6.1.4.1.1991."
local snChasGen = e_brocade .. "1.1.1.1"
local snChasPwr = e_brocade .. "1.1.1.2"
local snChasFan = e_brocade .. "1.1.1.3"
local snChasActualTemperature  = snChasGen .. ".18"
local snChasPwrSupplyOperStatus = snChasPwr .. ".1.1.3"
local snChasFanOperStatus = snChasFan .. ".1.1.3"

local function preWalkCB(ip)
   local agent,st = state.getState(mod,ip)
   st.temp = nil
   st.fan_count = 0
   st.fan_failed = 0
   st.psu_count = 0
   st.psu_failed = 0
end

local function walkCB(ip, status, rowid, temp, psuStatus, fanStatus)
   local agent,st = state.getState(mod,ip)
   if (status == nil) then
      -- temp provided in half-degrees
      local degrees = (tonumber(temp) or 0) / 2
      -- should only be one here, but take the max because we'll get nulls when there are multiple psu/fan entries
      measurement.accumulate(st, "temp", degrees, "max")
      local psu = tonumber(psuStatus) or 0
      if(psu == 1 or psu == 2 or psu == 3) then
        st.psu_count = st.psu_count + 1
        if(psu == 3) then st.psu_failed = st.psu_failed + 1 end
      end
      local fan = tonumber(fanStatus) or 0
      if(fan == 1 or fan == 2 or fan == 3) then
        st.fan_count = st.fan_count + 1
        if(fan == 3) then st.fan_failed = st.fan_failed + 1 end
      end
   elseif (status == "END") then
      logger.dbg(mod, agent, "walkCB normal termination, send ENV sample")
      measurement.env(ip, 0, st.temp, 0, 0, st.fan_count, st.fan_failed, st.psu_count, st.psu_failed)
   end
   return true -- keep going
end

return {
   mod = mod,
   objIds = { e_brocade },
   oids = { snChasActualTemperature, snChasPwrSupplyOperStatus, snChasFanOperStatus },
   preWalkCB = preWalkCB,
   walkCB = walkCB
}
