-----------------------------------------------------------------------
----- Copyright (C) InMon Corp. 2014 ALL RIGHTS RESERVED           ----
----- Module to poll CPU and memory stats from Brocade(TM) devices ----
-----------------------------------------------------------------------

local logger = require("logger")
local state = require("state")
local oid = require("oid")
local util = require("util")
local measurement = require("measurement")
local mod = "Brocade_LAG"

local e_brocade = "1.3.6.1.4.1.1991."
local e_brocade_lag = e_brocade .. "1.1.3.33.1.1.1."
local lag_ports = e_brocade_lag .. "4"
local lag_ifindex = e_brocade_lag .. "9"

local function preWalkCB(ip)
   local agent,st = state.getState(mod, ip)
   logger.dbg(mod, agent, "preWwalkCB")
   st.lags = {}
   st.num_lags = 0
end

local function walkCB(ip, status, rowid, lag_ports, lag_ifindex)
   if (status == nil) then
      local agent,st = state.getState(mod, ip)
      logger.dbg(mod, agent, "walkCB rowid=" .. rowid .. " lag_ports=" .. tostring(lag_ports) .. " lag_ifindex=" .. tostring(lag_ifindex))
      local lag = {}
      lag.actor = "000000000000"
      lag.partner = "000000000000"
      lag.ports = {}
      local ot = oid.oid_array(lag_ports)
      local ncomp = #ot / 4
      local start = 1
      aggID = tonumber(lag_ifindex)
      for ii = 1,ncomp do
          local pp = oid.be32_at(ot, start)
          start = start + 4
          lag.ports[ii] = pp
	  -- make up a new aggID for the case where lag_ifindex == first component
          if (pp == aggID) then aggID = 100000000 + st.num_lags + 1 end
      end
      st.lags[lag_ifindex] = lag
      st.num_lags = st.num_lags + 1
      logger.dbg(mod, agent, "lag=" .. util.table_print(lag, 3))
      if ncomp > 1 then
         for ii,pp in pairs(lag.ports) do
            measurement.lag(ip, pp, aggID, lag.actor, lag.partner)
         end
      end
   end
   return true -- keep going
end

return {
   mod = mod,
   objIds = { e_brocade },
   oids = { lag_ports, lag_ifindex },
   preWalkCB = preWalkCB,
   walkCB = walkCB
}
