 -----------------------------------------------------------------------
----- Copyright (C) InMon Corp. 2016 ALL RIGHTS RESERVED           ----
----- Module to poll Optical (DOM) stats from Brocade(TM) devices  ----
-----------------------------------------------------------------------

local measurement = require("measurement")
local logger = require("logger")
local state = require("state")
local util = require("util")
local oid = require("oid")

local mod = "Wlan_Aruba_AP"

local e_aruba = "1.3.6.1.4.1.14823."
local wlsxWlanAPEntry = e_aruba .. "2.2.1.5.2.1.4.1." -- index AP(mac)
local apIP = wlsxWlanAPEntry .. "2"
local apName = wlsxWlanAPEntry .. "3"

local function getAP(st, mac)
   if not st.aps then st.aps = {} end
   if not st.aps[mac] then st.aps[mac] = {
       mac=mac,
       ifindex=tonumber(mac:sub(5,12), 16) -- autogenerate ifindex from mac
   }
   end
   return st.aps[mac]
end

local function walkCB(ip, status, rowid, ap_ip, ap_name)
   if (status == nil) then
      local agent,st = state.getState(mod,ip)
      local ot = oid.oid_array(rowid)
      local ap_mac = oid.mac_at(ot, 1)
      local ap = getAP(st, ap_mac)
      ap.ip = ap_ip
      ap.name = ap_name
      logger.dbg(mod, agent, "rowid=" .. rowid .. " ip=" .. tostring(ap.ip) .. " ap_name=" .. tostring(ap.name) .. " ap_mac=" .. tostring(ap.mac) .. " ifindex=" .. tostring(ap.ifindex))
   end
   return true -- keep going
end

return {
   mod = mod,
   objIds = { e_aruba },
   oids = { apIP, apName },
   walkCB = walkCB
}
