 -----------------------------------------------------------------------
----- Copyright (C) InMon Corp. 2016 ALL RIGHTS RESERVED           ----
----- Module to poll Optical (DOM) stats from Brocade(TM) devices  ----
-----------------------------------------------------------------------

local measurement = require("measurement")
local logger = require("logger")
local state = require("state")
local util = require("util")
local oid = require("oid")
local ap_mod = require("pollWlan_Aruba_AP")

local mod = "Wlan_Aruba_APStats"

local e_aruba = "1.3.6.1.4.1.14823."
local wlsxWlanAPStatsEntry = e_aruba .. "2.2.1.5.3.1.1.1." -- index AP(mac).radio(int32).BSSID(mac)

local apPkts_rx = wlsxWlanAPStatsEntry .. "22"
local apBytes_rx = wlsxWlanAPStatsEntry .. "23"
local apPkts_tx = wlsxWlanAPStatsEntry .. "24"
local apBytes_tx = wlsxWlanAPStatsEntry .. "25"
local apErrs_retry = wlsxWlanAPStatsEntry .. "15"
local apErrs_chan = wlsxWlanAPStatsEntry .. "16"
local apErrs_rx = wlsxWlanAPStatsEntry .. "17"

local function getAP(st, mac)
   if not st.aps then return nil end
   return st.aps[mac]
end

local function preWalkCB(ip)
   local agent,st = state.getState(ap_mod.mod,ip)
   if st.aps then
      for mac,ap in pairs(st.aps) do
          ap.inoctets=0
          ap.outoctets=0
          ap.inucasts=0
          ap.outucasts=0
          ap.inerrors=0
          ap.outerrors=0
          ap.outdiscards=0
      end
   end
end

local function walkCB(ip, status, rowid, pkts_rx, bytes_rx, pkts_tx, bytes_tx, errs_rt, errs_ch, errs_rx)
   -- use sister module state
   local agent,st = state.getState(ap_mod.mod,ip)
   if (status == nil) then
      logger.dbg(mod, agent, "rowid=" .. rowid .. " pkts_rx=" .. tostring(pkts_rx))
      -- rowid has ap(mac) radio(int32) and BSSID(mac)
      local ot = oid.oid_array(rowid)
      local ap_mac = oid.mac_at(ot, 1)
      local ap = getAP(st, ap_mac)
      if ap then
         local radio = ot[7]
         local bssid =  oid.mac_at(ot,8)
         logger.dbg(mod, agent, "rowid=" .. rowid .. " ap=" .. ap.name .. " pkts_rx=" .. tostring(pkts_rx) .. " ap_mac=" .. tostring(ap_mac) .. " bssid=" .. tostring(bssid))
	 -- accumulate AP totals across all radio.bssid
	 measurement.accumulate(ap, "inoctets", bytes_rx, "sum")
	 measurement.accumulate(ap, "outoctets", bytes_tx, "sum")
	 measurement.accumulate(ap, "inucasts", pkts_rx, "sum")
	 measurement.accumulate(ap, "outucasts", pkts_tx, "sum")
	 measurement.accumulate(ap, "inerrors", errs_rx, "sum")
	 measurement.accumulate(ap, "outerrors", errs_ch, "sum")
	 measurement.accumulate(ap, "outdiscards", errs_rt, "sum")
      end
   elseif (status == "END") then
       -- send results
       if st.aps then
         for mac,ap in pairs(st.aps) do
          measurement.if_counters_xdr(agent.ip,
	                             ap.ifindex,
                                     71, -- 802.11
                                     0,  -- ifSpeed unknown
                                     0, -- direction half/full duplex unknown
                                     3, -- status oper-up, admin-up
                                     ap.inoctets,
                                     ap.inucasts,
                                     0, -- ap.inmcasts
				     0, -- ap.inbcasts
				     ap.indiscards,
				     ap.inerrors,
				     0, -- ap.unknownprotos
				     ap.outoctets,
				     ap.outucasts,
				     0, -- ap.outmcasts
				     0, -- ap.outbcasts
				     ap.outdiscards,
				     ap.outerrors,
				     1, -- promisc
				     ap.name)
             -- pass in more meta-data
	     -- measurement.iftable(agent.ip, ap.ifindex, { ifName=ap.name, ifDescr="Aruba AP", ifAlias=ap.ip, ifPhysAddr=ap.mac })
         end
       end
   end
   return true -- keep going
end

return {
   mod = mod,
   objIds = { e_aruba },
   oids = { apPkts_rx, apBytes_rx, apPkts_tx, apBytes_tx, apErrs_retry, apErrs_chan, apErrs_rx },
   preWalkCB = preWalkCB,
   walkCB = walkCB
}
