-----------------------------------------------------------------------
----- Copyright (C) InMon Corp. 2014 ALL RIGHTS RESERVED       --------
----- Module to poll CPU and memory stats from                 --------
----- Dell/Force10 E-Series devices                            --------
-----------------------------------------------------------------------

local measurement = require("measurement")
local logger = require("logger")
local state = require("state")
local mod = "F10_E"

local e_f10e = "1.3.6.1.4.1.6027.1.1." -- E Series

local f10_chassis = "1.3.6.1.4.1.6027.3.1"

-- chRpmCpuUtil1Min[chRpmCpuIndex]
-- rpm = "route processing module"
local f10_cpu60_rpm = f10_chassis .. ".1.3.7.1.4"

-- chRpmMemUsageUtil[chRpmCpuIndex]
local f10_memutil_rpm = f10_chassis .. ".1.3.7.1.6"

-- chLineCardCpuUtil1Min[chSysCardNumber]
local f10_cpu60_lc = f10_chassis .. ".1.5.1.1.2"

-- chLineCardMemUsageUtil[chSysCardNumber]
local f10_memutil_lc = f10_chassis .. ".1.5.1.1.4"

-- chSysProcessorMemSize[chSysSwSlotIndex,chSysSwProcessorIndex]
local f10_mem_total = f10_chassis .. ".1.2.6.1.6"

-- to match memutil with memtotal requires index mappings from
-- the chSysCardTable.  Here slots 0,1 are rpms, and
-- slots 2..N are line-cards.  Knowing slot>card>processor
-- allows us to join the data from the other tables.  We get
-- temperature info from this table too.
local f10_cardtable = f10_chassis .. ".1.2.3"
local f10_slot = f10_cardtable .. ".1.1"
local f10_card = f10_cardtable .. ".1.3"
local f10_temp_max = f10_cardtable .. ".1.8"
local f10_temp_min = f10_cardtable .. ".1.9"
local f10_card_processors = f10_cardtable .. ".1.10"

-- but for now we just take the max cpuutil and
-- max memutil across any rpm processor or
-- line-card.

local function preWalkCB(ip)
   local agent,st = state.getState(mod, ip)
   st.mem_total = 0
   st.cpu60 = 0
   st.memutil = 0
   st.temp_min = nil
   st.temp_max = nil
end

local function walkCB(ip, status, rowid, mem_total, cpu60, cpu60_lc, memutil_rpm, memutil_lc, temp_max, temp_min)
   local agent,st = state.getState(mod, ip)
   if (status == nil) then
      measurement.accumulate(st, "mem_total", mem_total, "sum")
      measurement.accumulate(st, "cpu60", cpu60, "max")
      measurement.accumulate(st, "cpu60", cpu60_lc, "max")
      measurement.accumulate(st, "memutil", memutil_rpm, "max")
      measurement.accumulate(st, "memutil", memutil_lc, "max")
      measurement.accumulate(st, "temp_max", temp_max, "max")
      measurement.accumulate(st, "temp_min", temp_min, "min")
   elseif (status == "END") then
      measurement.processor(ip, st.mem_total, 0, st.cpu60)
      measurement.processorMax(ip, st.cpu60, st.memutil)
      measurement.env(ip, 0, st.temp_max, 0, 0)
      measurement.rtmetric(ip, 3, 1, "entity.temp_min", 1, "gaugedouble", st.temp_min)
   end
   return true -- keep going
end

return {
   mod = mod,
   objIds = { e_f10e },
   oids = { f10_mem_total, f10_cpu60_rpm, f10_cpu60_lc, f10_memutil_rpm, f10_memutil_lc, f10_temp_max, f10_temp_min },
   preWalkCB = preWalkCB,
   walkCB = walkCB
}


