-----------------------------------------------------------------------
----- Copyright (C) InMon Corp. 2014 ALL RIGHTS RESERVED       --------
----- Module to poll CPU and memory stats from                 --------
----- Dell/Force10 S-Series devices                            --------
-----------------------------------------------------------------------

local measurement = require("measurement")
local logger = require("logger")
local state = require("state")
local mod = "F10_S"

local e_f10s = "1.3.6.1.4.1.6027.1.3." -- S Series

local f10_chassis = "1.3.6.1.4.1.6027.3.10"

local f10_stackunit = f10_chassis .. ".1.2.2"
local f10_power = f10_chassis .. ".1.2.3"
local f10_fan = f10_chassis .. ".1.2.4"
local f10_processor = f10_chassis .. ".1.2.7"
local f10_stackunit_util = f10_chassis .. ".1.2.9"

-- chStackUnitCpuUtil1Min[chStackUnitNumber]
local f10_cpu60 = f10_stackunit_util .. ".1.3"

-- chStackUnitMemUsageUtil[chStackUnitNumber]
local f10_memutil = f10_stackunit_util .. ".1.5"

-- chSysProcessorMemSize[chStackUnitNumber]
local f10_mem_total = f10_processor .. ".1.4"

-- chStackUnitTemp[chStackUnitIndex]
local f10_temp = f10_stackunit .. ".1.14"
local f10_fan_count = f10_stackunit .. ".1.27"
local f10_psu_count = f10_stackunit .. ".1.28"

-- chSysPowerSupplyOperStatus[chStackUnitIndex,chSysPowerSupplyIndex]
local f10_psu_status = f10_power .. ".1.2"
-- chSysFanTrayOperStatus[chStackUnitIndex,chSysFanTrayIndex]
local f10_fan_status = f10_fan .. ".1.2"

-- memutil and mem_total are both indexed by chStackUnitNumber
-- even though they are in different tables,  so theoretically
-- we can extract the free_memory numbers that way and populate
-- the sFlow processor structure with it.  However for now we
-- just offer the max memutil.

local function preWalkCB(ip)
   local agent,st = state.getState(mod, ip)
   st.mem_total = 0
   st.cpu60 = 0
   st.memutil = 0
   st.temp_min = nil
   st.temp_max = nil
   st.fan_count = 0;
   st.fan_failed = 0
   st.psu_count = 0
   st.psu_failed = 0
end

local function walkCB(ip, status, rowid, mem_total, cpu60, memutil, temp, fan_count, fan_status, psu_count, psu_status)
   local agent,st = state.getState(mod, ip)
   if (status == nil) then
      measurement.accumulate(st, "mem_total", mem_total, "sum")
      measurement.accumulate(st, "cpu60", cpu60, "max")
      measurement.accumulate(st, "memutil", memutil, "max")
      measurement.accumulate(st, "temp_max", temp, "max")
      measurement.accumulate(st, "temp_min", temp, "min")
      measurement.accumulate(st, "fan_count", fan_count, "sum")
      if(fan_status == "2") then
	 measurement.accumulate(st, "fan_failed", 1, "sum")
      end
      measurement.accumulate(st, "psu_count", psu_count, "sum")
      if(psu_status == "2") then
	 measurement.accumulate(st, "psu_failed", 1, "sum")
      end
   elseif (status == "END") then
      measurement.processor(ip, st.mem_total, 0, st.cpu60)
      measurement.processorMax(ip, st.cpu60, st.memutil)
      measurement.env(ip, 0, st.temp_max, 0, 0, st.fan_count, st.fan_failed, st.psu_count, st.psu_failed)
      measurement.rtmetric(ip, 3, 1, "entity.temp_min", 1, "gaugedouble", st.temp_min)
   end
   return true -- keep going
end
return {
   mod = mod,
   objIds = { e_f10e },
   oids = { f10_mem_total, f10_cpu60, f10_memutil, f10_temp, f10_fan_count, f10_fan_status, f10_psu_count, f10_psu_status },
   preWalkCB = preWalkCB,
   walkCB = walkCB
}
