-----------------------------------------------------------------------
----- Copyright (C) InMon Corp. 2014 ALL RIGHTS RESERVED       --------
----- Module to collect extra metrics from agents              --------
-----------------------------------------------------------------------
local util = require("util")
local oid = require("oid")
local state = require("state")
local logger = require("logger")
local mod = "POLL"
local interval = inx.pref("@SNMPCounterPollInterval") or 60
interval = tonumber(interval)

local pollers = {
   require("pollCpuMem_HR"),
   require("pollCpuMem_HP"),
   require("pollCpuMem_Cisco"),
   require("pollCpuMem_Brocade"),
   require("poll_Force10_E"),
   require("poll_Force10_S"),
   require("pollEnv_EntitySensors"),
   require("pollOptical_Brocade"),
   require("pollWlan_Aruba_APStats"),
   require("pollWlan_Aruba_AP"),
   -- require("pollLAG_Brocade_LACP"),
   -- require("pollLAG_Brocade"),
}

local function sendSNMP(ctx)
   local agent = ctx.agent
   local poller = ctx.poller
   local mod = ctx.mod
   logger.dbg(mod, agent, "sendSNMP")
   inx.timer(sendSNMP, ctx, agent.poll_mS)
   if(agent.up) then
      -- TWC might have to be rebuilt if settings changed
      if not inx.TWContextOK(ctx.twc) then
	 if(poller.tw and not ctx.twc) then
	    logger.dbg(mod, agent, "build TWC")
	    ctx.twc = inx.TWContextCreate(poller.tw, agent.ip)
	 end
      end
      if(poller.getCB) then
	 logger.dbg(mod, agent, "send GET")
	 inx.TWDoGet2(ctx.twc, poller.getCB)
      end
      if(poller.walkCB) then
	 if(poller.preWalkCB) then
	    logger.dbg(mod, agent, "preWalkCB")
	    poller.preWalkCB(agent.ip)
	 end
	 logger.dbg(mod, agent, "start WALK")
	 inx.TWDoTableWalk(ctx.twc, poller.walkCB)
      end
   end
end

local function doAgent(ip, objId, up, args)
   local agent = state.getAgent(ip)
   -- write these settings at the agent top level
   agent.up = up
   agent.objId = objId
   agent.poll_mS = 1000 * interval
   logger.dbg(mod, agent, "doAgent: objId=" .. tostring(objId) .. " up=" .. tostring(up))
   for i,poller in ipairs(pollers) do
      -- must have a mod string, so make one up if it not set
      if(poller.mod == nil) then poller.mod = "poller" .. tostring(i) end
      -- test sysObjectID to see if this poller wants to poll this agent
      if(oid.objIdMatch(objId, poller.objIds)) then
	 if(poller.agentCB) then
	    -- simple callback to tell poller about agent
	    poller.agentCB(ip, objId, up, args)
	 end
	 if(poller.oids and (poller.getCB or poller.walkCB)) then
	    -- poller want periodic callback with results of GET or WALK 
	    -- first make sure the tablewalk is there at the poller level
	    if not poller.tw then
	       logger.dbg(mod, agent, "build TW")
	       poller.tw = inx.TWCreate(poller.oids, "(mod=" .. tostring(poller.mod) .. ")")
	    end
            if poller.tw then
               -- OK, now check for the per-poller,per-agent polling context.
	       -- Be careful to remember if we have already scheduled this.
	       local ag,ctx = state.getState(poller.mod, ip)
               if not ctx.scheduled then
                 ctx.scheduled = true
                 ctx.poller = poller
	         inx.timer(sendSNMP, ctx, math.random(agent.poll_mS))
               end
            end
	 end
      end
   end
end

local function agentsChanged(args)
   logger.log1(mod, nil, "Got callback: agentsChanged")
   inx.agentsDo(doAgent, args)
end

inx.sinkEvent("agentsChanged", agentsChanged, {} )

local listeners = {
   -- require("listen_VCS"),
}

local poller = { pollers=pollers, listeners=listeners, state=state }
return poller
