// path-filter: nav.hosts

var webapp = WebApp.current();
var chartSpec = webapp.getValue("com.inmon.servlets.hosts.trendcharts");
var trends = JSON.parse(chartSpec);
var path = webapp.getValue("com.inmon.servlets.hosts.path");
var machineType = webapp.getValue("com.inmon.servlets.hosts.machineType");
var osName = webapp.getValue("com.inmon.servlets.hosts.osName");
var show = webapp.getValue("com.inmon.servlets.hosts.trendshow");
var interval = webapp.getValue("com.inmon.servlets.hosts.interval");
var refresh = "true" == webapp.getValue("com.inmon.servlets.hosts.refresh");
var group = "minute";

var n = Network.current();
n.path = path;

Date.prototype.toJSON = function() { return formatdate(this,'yyyy/MM/dd HH:mm:ss'); };

var showAllFlag =  !show || "all" == show;

function localize(str) {
   var res = webapp.localize("t." + str);
   if(res.indexOf("t.") === 0) {
     res = webapp.localize("c." + str);
     if(res.indexOf("c.") === 0) res = str;
   }
   return res;
}

function columnLabel(cname,localizeFlag) {
  // strip aggregations function to get underlying column name
  var match = /\(([^\s]+)\)/.exec(cname);
  var label =  match ? match[1] : cname;

  // get part of name after dot
  var idx = label.lastIndexOf('.');
  if(idx >= 0) label = label.substring(idx+1);
  return localizeFlag ? localize(label) : label;
}

function createChart(t,trend,status,title) {

  // fix column names
  for(var c = 0; c < t.cnames.length; c++) t.cnames[c] = columnLabel(t.cnames[c],true);

  // scale data, if needed
  if(trend.scale > 0) {
     for(var c = 1; c < t.ncols; c++) t.scaleColumn(c,trend.scale);
  }

  var chart = Chart.multiSeries(
                      "trend",
                      title,
                      t,
                      null,
                      0,
                      localize(trend.ylabel),
                      Chart.range(1,t.ncols - 1));
  chart.step = true;
  chart.stack = trend.stack;
  if(trend.includeMillis) chart.includeMillis = true;
  if(trend.colors) chart.colors = trend.colors;
  return chart;
}

function data() {
  var charts = [];

  var filter = "path=" + (n.isVirtualHost() ? "'" + n.parent() + "'&v_dsindex=" + n.dsIndex() : "'" + path + "'");
  if(machineType) filter += " & machine_type=" + machineType;
  if(osName) filter += " & os_name=" + osName;


  if(showAllFlag) {
    // build all the charts at this level in the path

    if(!n.isVirtualHost()) {
       // filter the list of trends to exclude virtual stats
       trends = trends.filter(function(trend) { return !trend.virtual; });
    } else {
       if("java" == n.hostOSName()) {
          // filter out the counters that JVM doesn't support
          trends = trends.filter(
             function(trend) {
                var id = trend.id;
                if(!trend.virtual) return true;
 
                var keep;
                switch(id) {
                  case 'viooperations':   keep = false; break;
                  case 'viobytes':        keep = false; break;
                  case 'vnetworkbytes':   keep = false; break;
                  case 'vnetworkpackets': keep = false; break;
                  default: keep = true;
                }
                return keep;
             }
          );
       } else {
         // filter out the JVM specific counters
         trends = trends.filter(
            function(trend) {
               var id = trend.id;
               if(!trend.virtual) return true;
               var keep;
               switch(id) {
                 case 'jvmheap'         : keep = false; break;
                 case 'jvmnonheap'      : keep = false; break;
                 case 'jvmthreads'      : keep = false; break;
                 case 'jvmgc'           : keep = false; break;
                 case 'jvmfd'           : keep = false; break; 
                 case 'jvmthreadstarted': keep = false; break;
                 default: keep = true;
               }
               return keep;
            }
          );
       }
    }

    var select = [];

    var q = Query.trend("host", select, filter, interval, group);
    q.multiquery = true;

    for(var i = 0; i < trends.length; i++) {
      select.push(trends[i].select);
    }

    var tables = q.run();

    // override - these charts are layed out for two columns
    for(var i = 0; i < trends.length; i++) {
      var t = tables[i];
      var status = "unknown";

      if(t) {
        var chart = createChart(t,trends[i],status,webapp.localize("hosts.trend.show." + trends[i].id));
        chart.nodeClick = {id: trends[i].id};
        charts.push(chart);
      }
    }
  }
  else {
    // build the same chart for every child in this path

    var truncate = webapp.getValue("com.inmon.servlets.hosts.trendTruncate");

    var trend = null;
    for(var i = 0; i < trends.length; i++) {
      if(trends[i].id == show) {
         trend = trends[i];
         break;
       }
    }
    if(trend) {
      var ppath = path.split(">");

      // don't make query if we are looking at a physical data source and the chart is virtual
      if(ppath.length == 6 && trend.virtual && !n.isVirtualHost()) return;
      
      var childkey;
      switch(ppath.length) {
        case 2: childkey = "zone"; break;
        case 3: childkey = "group"; break;
        case 4: childkey = "agent"; break;
        case 5: childkey = trend.virtual ? "v_dsindex" : "interface"; break;
        case 6: childkey = trend.virtual ? "v_dsindex" : "interface"; break;
        default: childkey = "zone";
      }
 
      // make a top N query to find the set of paths
      var children = [];
      var filters = [];
      
      // need to put in the select keys, sort key may contain function
      var select = trend.select.split(',');
      // remove functions
      for(var i = 0; i < select.length; i++) select[i] = columnLabel(select[i],false);
      // replace time with childkey
      select[0] = childkey;
      select.push(trend.sort);

      var q = Query.topN("host",select, filter + ("& " + (trend.virtual ? "has_vm = 1" : "has_host = 1")),interval,trend.sort,truncate);
      q.run(function(row) {
         if(row[0]) {
            children.push(row[0]);
            filters.push(filter + "&" + childkey + '="' + row[0] + '"');
         }
      });

      // build a multiquery for each of the trends
      var tables;
      if(filters.length > 0) {
        q = Query.trend("host",trend.select,filters,interval,group);
        q.multiquery = true;
        tables = q.run(); 
      }

      if(tables) {
        var names;
        if("interface" == childkey) {
           names = n.hostIdToNameMap(children);
           for (var i = 0; i < children.length; i++) {
             if(!names[i]) names[i] = children[i];  
           }
         }
         
         for(var i = 0; i < tables.length; i++) {
           var child = children[i];
           var ct = tables[i];

           var title = child;
           if("interface" == childkey) title = names[i];
           else if("v_dsindex" == childkey) {
              n.path = ppath.length == 6 ? path : path + ">3:" + child;
              title = n.hostName() ? n.hostName() : n.interfaceId();
           }
           else if("agent" == childkey) {
              n.path = child;
              title = n.displayName();
           }
           else if("EXTERNAL" == child) {
              // Other zone or group
              title = webapp.localize("monitor.other");
           }
           var status = "unknown";
           var chart = ppath.length < 6 ? createChart(ct,trend,status,title) : createChart(ct,trend,status,title,null,null);  
           var cpath = childkey == "interface" ? child : (childkey == "v_dsindex" ? n.path : path + ">" + ("EXTERNAL" != child ? child : ""));
           chart.nodeClick = {id: trend.id, value: cpath};
           charts.push(chart);
        }
      }
    }
  }
  println(JSON.stringify({data:charts}));
}

function action(evtName,evtId,evtValue) {
  switch(evtName) {
    case "nodeClick":
      var response = {action:"loadpage"};
      var urlArgs = {};
      if(evtValue) urlArgs.path=evtValue;
      else urlArgs.show=evtId;
      response.url = webapp.encodeRedirectURL(webapp.createURL(urlArgs));
      println(JSON.stringify(response,null,2));
      break;
    default: data();
  }
  
}

webapp.charts(null,action,{
   refresh    : refresh,
   columns    : showAllFlag ? 2 : 0,
   rebuild    : !showAllFlag,
   large      : n.isInterface() && !showAllFlag,   
   dataTimeout: preference("ajax.timeout") ? preference("ajax.timeout") : 600000
});
