// path-filter: nav.map:nav.map.l3

var value = 'frames';
var view = 'rttraffic';
var cshape = "ellipse";
var ccolor = "LIGHT_BLUE";


var webapp = WebApp.current();

var n = Network.current();

function colorForStatus(status) {
    var color;
    switch(status) {
    case "severe":
	color = "RED";
	break;
    case "warn":
	color = "YELLOW";
	break;
    case "good":
	color = "LIGHT_GREEN";
	break;
    case "unknown":
	color = "LIGHT_GRAY";
	break;
    default:
	color = "LIGHT_GRAY";
    }
    return color;
}
	
function createmap() {
  var allCIDRs = webapp.getValue("cidr") != "transit";

  var graph = {};
  var nodes = {};
  var edges = {};
  graph.nodes = nodes;
  graph.edges = edges;

  function calculateLinkWeights(adj) {
    var show = webapp.getValue("show","links");

    var weights = {};

    var maxval = 0;

    switch(show) {
    case "ifSpeed": 
        for(var if1 in adj) {
          n.path = if1;
          var ifSpeed = n.ifSpeed();
          n.path = adj[if1];
          ifSpeed = Math.max(ifSpeed,n.ifSpeed());
          weights[if1] = ifSpeed;
          maxval = Math.max(maxval,ifSpeed);
        }
        break;
    case "bps": 
        var ifs = [];
        for (var if1 in adj) ifs.push(if1);
        if(ifs.length) {
          var q = Query.topN("rtcounters","interface,ifinoctets,ifoutoctets","interface=" + ifs,webapp.getValue("interval","last15minutes"),null,null);
          q.run(
           function(row) { 
              var val = row[1] + row[2];
              weights[row[0]] = val;
              maxval = Math.max(maxval,val);
           }
          );
        }
        break;
    default:
        maxval = 5.0;
        for(var if1 in adj) weights[if1] = maxval;
    }

    // normalize the weights
    for(var if1 in weights) {        
       weights[if1] = Math.max(1,Math.round(Math.log(100 * weights[if1] / maxval)));
    }

    return weights;
  }

  function addAgent(name,address,status) {
     if(!nodes[address]) {
        nodes[address] = {
            label: name, 
            color: colorForStatus(status),
            evtId: 'agent'
        }
     }
  }
  function addCIDR(cidr) {
     if(!nodes[cidr]) {
        nodes[cidr] =  {
            label:  cidr, 
            shape: cshape, 
	    color: ccolor,
            evtId: 'cidr'
        }
     }
  }
  function addEdge(id,node1,node2,weight,status) {
     if(!edges[id]) {
        edges[id] = {
           node1: node1,
           node2: node2,
           evtId: 'edge'
        }
        if(weight) {
           edges[id].weight = weight;
           if(status) edges[id].color = colorForStatus(status);
        }
        else {
           edges[id].lineType = "dotted";
        }
     }
  }
  function includeCIDR(cidr) {
    if(cidr.match("/32$")) return false;
    return true;
  }

  var ifs = n.allInterfaces();
  var cidrMap = n.interfaceToDeviceCidrMap(ifs);
  var ifToCidrs = {};
  var agents = {};
  var cidrs = {};
  for(var i = 0; i < ifs.length; i++) {
    if(cidrMap[i]) {
      var ifx = ifs[i];
      n.path = ifx;
      var ip = n.agentIP();
      if(!agents[ip]) agents[ip] = [ifx];
      else agents[ip].push(ifx);

      var cidrList = cidrMap[i];
      for(var j = 0; j < cidrList.length; j++) {
         var cidr = cidrList[j];
         if(includeCIDR(cidr)) {
           if(!cidrs[cidr]) cidrs[cidr] = 1;
           else cidrs[cidr]++;
         }
      }
      ifToCidrs[ifx] = cidrList;
    }
  }

  var status = {};
  var agentlist = [];
  for (var ip in agents) {
    if(agents[ip].length > 1) agentlist.push(ip);
  }
  var statuslist = n.statusMap(agentlist,["status"]);
  for (var i = 0; i < agentlist.length; i++) {
     if(statuslist[i] && statuslist[i].length > 0) {
       status[agentlist[i]] = statuslist[i][0];
     }
  }

  for (var ip in agents) {
       if(agents[ip].length > 1) {
         n.path = ip;
         addAgent(n.displayName(),ip,status[ip]);
       }
   }
      
   for (var cidr in cidrs) {
       if(allCIDRs || cidrs[cidr] > 1) addCIDR(cidr);
   }

   var adj = {};

   for (var ip in agents) {
      var ifList = agents[ip];
      if(ifList.length > 1) {
        for(var i = 0; i < ifList.length; i++) {
          var ifx = ifList[i];
          var cidrList = ifToCidrs[ifx];
          for(var j = 0; j < cidrList.length; j++) {
            var cidr = cidrList[j];
            if((allCIDRs && cidrs[cidr]) || cidrs[cidr] > 1) adj[ifx] = true;
          }
        }
      }
   }

   var statusFlag = webapp.getValue("show","status") == "status";
   status = {};
   if(statusFlag) {
      var portlist = [];
      for (var ip in agents) {
        var ifList = agents[ip];
        if(ifList.length > 1) {
          for(var i = 0; i < ifList.length; i++) {
            var ifx = ifList[i];
            var cidrList = ifToCidrs[ifx];
            for(var j = 0; j < cidrList.length; j++) {
              var cidr = cidrList[j];
              if((allCIDRs && cidrs[cidr]) || cidrs[cidr] > 1) {
                 portlist.push(ifx);
              }	
            }
          }
        }
      }
      statuslist = n.statusMap(portlist,["link"]);
      for (var i = 0; i < portlist.length; i++) {
        if(statuslist[i] && statuslist[i].length > 0) {
          status[portlist[i]] = statuslist[i][0];
        }
      }
   }

   var weights = calculateLinkWeights(adj);

   for (var ip in agents) {
      var ifList = agents[ip];
      if(ifList.length > 1) {
        for(var i = 0; i < ifList.length; i++) {
          var ifx = ifList[i];
          var cidrList = ifToCidrs[ifx];
          for(var j = 0; j < cidrList.length; j++) {
            var cidr = cidrList[j];
            if((allCIDRs && cidrs[cidr]) || cidrs[cidr] > 1) {
               n.path = ifx;
               addEdge(ifx + "," + cidr, ip, cidr, weights[ifx], status[ifx]);
            }
          }
        }
      }
   }

   return graph;
}

function header() {  
  var show = webapp.queryParameter("show",webapp.getValue("show"));
  var filter = [ 
         webapp.button(webapp.localize("map.reset"),webapp.createURL({host:"all"}))
             ];
  var subfilter = [
         webapp.spinBox("show","t.show","status",["status","ifSpeed","bps"],[webapp.localize("map.show.status"),webapp.localize("map.show.ifSpeed"),webapp.localize("map.show.bps")]),
         webapp.spinBox("cidr",webapp.localize("map.cidr"),"all",["all","transit"],[webapp.localize("map.cidr.all"),webapp.localize("map.cidr.transit")])
                 ];
   if("bps" == show) subfilter.push(webapp.spinBox("interval","t.interval","last15minutes","interval"));
   webapp.filterBar(filter,subfilter);

}

function data(evtName, evtId, evtValue) {

  var resp = {};

  switch(evtName) {
    case "nodeClick": 
       switch(evtId) {
       case "agent":n.path = evtValue;
            resp.action = "dialog";
            resp.title = n.agent();
            resp.values = [];
            resp.values.push({label:webapp.localize("map.enterprise"),value:n.enterpriseName()});
            resp.values.push({label:webapp.localize("config.zone"),value:n.zone()});
            resp.values.push({label:webapp.localize("config.group"),value:n.group()});
            resp.values.push({label:webapp.localize("map.search"),value:evtValue,evtId:'searchAgent'});
            break;
       case "cidr":
            resp.action = "dialog";
            resp.title = webapp.localize("c.subnet");
            resp.values = [];
	    resp.values.push({label:webapp.localize("map.search"),value:evtValue,evtId:'searchSubnet'});
            break;
       }
       break;
    case "edgeClick": 
	switch(evtId) {
	case "edge":
	    if(evtValue) {
		resp.action = "dialog";
		resp.title = webapp.localize("map.link");
		var parts = evtValue.split(",");
		var agent = parts[0];
		var cidr = parts[1];
		resp.values = [];
		resp.values.push({label:webapp.localize("map.search"),value:agent,evtId:'searchAgent'});
		resp.values.push({label:webapp.localize("map.search"),value:cidr,evtId:'searchSubnet'});
	    }
	    break;
	}
       break;
    case "dialogClick":
        switch(evtId) {
        case "searchAgent":
	    resp.action = "loadpage";
	    resp.url = webapp.encodeRedirectURL('Go?action=agent&agent=' + evtValue);
            break;
	case "searchSubnet":
	    resp.action = "loadpage";
	    resp.url = webapp.encodeRedirectURL('Go?action=host&host=' + evtValue);
	    break;
	}
        break;
    default:
       resp.action = "loadgraph";
  }

  if(resp.action == "loadgraph") {
    resp.graph = createmap();
  }

  println(JSON.stringify(resp, null, 2)); 
}

webapp.graph(
	     header, 
	     data, 
	     {
		     layout: "organic",
		     refresh: true,
                     height: webapp.getValue("com.inmon.servlets.map.height"),
		     panLabel: webapp.localize("map.pan"),
		     upLabel: webapp.localize("map.pan.up"),
		     downLabel: webapp.localize("map.pan.down"),
		     leftLabel: webapp.localize("map.pan.left"),
		     rightLabel: webapp.localize("map.pan.right"),
		     zoomLabel: webapp.localize("map.zoom"),
		     inLabel: webapp.localize("map.zoom.in"),
		     outLabel: webapp.localize("map.zoom.out"),
		     layoutLabel: webapp.localize("map.layout"),
		     organicLabel: webapp.localize("map.layout.organic"),
		     orthogonalLabel: webapp.localize("map.layout.orthogonal"),
		     treeLabel: webapp.localize("map.layout.tree"),
		     circularLabel: webapp.localize("map.layout.circular"),
		     hierarchicLabel: webapp.localize("map.layout.hierarchic"),
		     resetZoomLabel: webapp.localize("map.zoom.reset")
	      });
