// path-filter: nav.map:nav.map.path
var performance=[];
function now_ms() { return (new Date).getTime(); }
var tcursor=now_ms();
function checkpoint(step) {
    var now = now_ms();
    performance.push("" + (now - tcursor));
    performance.push("" + step);
    tcursor = now;
}
checkpoint("start");

var webapp = WebApp.current();

var n = Network.current();

var hcolor = "LIGHT_CORAL";

function accumulateMax(htable, key, val) {
    var x = htable[key] || 0;
    htable[key] = Math.max(x, val);
}

function ifx_to_IP(ifx) {
    if(ifx) {
	var parts = ifx.split(">");
	if(parts.length == 2) return parts[0];
    }
    return null;
}

function colorForStatus(status) {
    var color;
    switch(status) {
    case "severe":
	color = "RED";
	break;
    case "warn":
	color = "YELLOW";
	break;
    case "good":
	color = "LIGHT_GREEN";
	break;
    case "unknown":
	color = "LIGHT_GRAY";
	break;
    default:
	color = "LIGHT_GRAY";
    }
    return color;
}

function createMap(source,destination) {	

  var graph = {};
  var nodes = {};
  var edges = {};
  graph.nodes = nodes;
  graph.edges = edges;
  //graph.exteriorNodeLabels = true;

  function trafficFilter() {
    var filt = null;

    var host = webapp.getValue("host");
    if(host) {
      var atype = n.addressType(host);

      var srccol = 'sourceaddress';
      var dstcol = 'destinationaddress';
      switch(atype) {
        case 'uuid': srccol = 'uuidsource'; dstcol = 'uuiddestination'; break;
        case 'ipv4': srccol = 'ipsource';  dstcol = 'ipdestination';  break;
        case 'ipv6': srccol = 'ip6source'; dstcol = 'ip6destination'; break;
        case 'mac' : srccol = 'macsource'; dstcol = 'macdestination'; break;
      }
      var host_filter = srccol + '=' + host + "|" + dstcol + '=' + host;

      filt = "(" + host_filter + ")";
    }

    var protocol = webapp.getValue("protocol");
    if(protocol) {
      var parts = protocol.split(":");
      var ptype = parts[0];
      var pnum = parts[1];
      var protocol_filter;
      switch(ptype) {
        case 'ETHERNET': protocol_filter = "ethernetprotocol=" + pnum; break;
        case 'IEEE802' : protocol_filter = "ieee802sap=" + pnum; break;
        case 'IP'      : protocol_filter = "ipprotocol=" + pnum; break;
        default: protocol_filter = "serverport=" + protocol;
      }
      if(filt) filt += "&" + protocol_filter;
      else filt = protocol_filter;
    }

    var where = webapp.getValue("where");
    if(where) {
       if(filt) filt += "&(" + where + ")";
       else filt = where;
    }
    
    return filt;
  }

  function normalizeWeights(weights,maxval) {
    checkpoint("normalizeWeights");
    for(var ifx in weights) {        
       weights[ifx] = Math.max(1,Math.round(Math.log(100 * weights[ifx] / maxval)));
    }
  }

  function calculateLinkWeights(adj) {
      checkpoint("calculateLinkWeights");

    var show = webapp.getValue("show","status");

    var weights = {};

    var maxval = 0;

    switch(show) {
    case "ifSpeed": 
        for(var if1 in adj) {
          n.path = if1;
          var ifSpeed = n.ifSpeed();
          n.path = adj[if1];
          ifSpeed = Math.max(ifSpeed,n.ifSpeed());
          weights[if1] = ifSpeed;
          maxval = Math.max(maxval,ifSpeed);
        }
        break;
    case "bps": 
        var ifs = [];
        for (var if1 in adj) ifs.push(if1);
        if(ifs.length) {
          var q = Query.topN("rtcounters","interface,ifinoctets,ifoutoctets","interface=" + ifs,webapp.getValue("interval","last15minutes"),null,null);
          q.run(
           function(row) { 
              var val = row[1] + row[2];
              weights[row[0]] = val;
              maxval = Math.max(maxval,val);
           }
          );
        }
        break;
    case "traffic":
        var ifs = [];
        for(var if1 in adj) ifs.push(if1);
        if(ifs.length) {
            var where = trafficFilter();
	    if(where) where = "(" + where + ") & path=" + ifs;
            else where = "path=" + ifs;
	    // take advantage of the fact that for unicast traffic we may have both ingress
	    // and egress interfaces identified
            var q = Query.topN("rtfanout",["inputinterface,llbytes", "outputinterface,llbytes"],where, webapp.getValue("interval","last15minutes"),null,null);
	    q.multiquery=true;
            q.run(
		function(row) {
		    var ifx = row[0];
		    if(ifx) {
			var val = row[1];
			// interface weight is max(in/out) -- partly to guard against double-counting in the event of bidirectional sampling
			accumulateMax(weights, ifx, val);
			maxval = Math.max(maxval,val);
		    }
		}
            );
        }
        break;
    default:
        maxval = 5.0;
        for(var if1 in adj) weights[if1] = maxval;
    }

    normalizeWeights(weights,maxval);

    return weights;
  }

  function addAgent(name,address,status) {
     if(!nodes[address]) {
        nodes[address] = {
            label: name, 
            color: colorForStatus(status),
	    evtId: 'agent'
        }
     }
  }

  function addHost(name,address) {
     if(!nodes[address]) {
        nodes[address] = {
            label: name, 
            shape: "rectangle",
            color: hcolor,
	    evtId: 'host',
	    evtValue: address + "," + name
        };
     }
  }

  function addEdge(id,node1,node2,weight,status) {
     if(!edges[id]) {
        edges[id] = {
           node1: node1,
           node2: node2,
	   evtId: 'edge'
        };
        if(weight) {
           edges[id].weight = weight;
	   if(status) edges[id].color = colorForStatus(status);
        }
        else {
           edges[id].lineType = "dotted";
        }
     }
  }

  function addHostEdge(id,node1,node2,weight,status) {
     if(!edges[id]) {
        edges[id] = {
           node1: node1,
           node2: node2,
	   evtId: 'hostEdge'
        }
        if(weight) {
           edges[id].weight = weight;
           if(status) edges[id].status = status;
        }
        else {
           edges[id].lineType = "dotted";
           edges[id].color = "#cccccc";
        }
     }
  }

  function interfaceAdjacency() {
      var ifAdj = {};
      var ifs = n.interfaces();
      var neighbors = n.neighborMap(ifs);
      for(var i = 0; i < ifs.length; i++) {
	  if(neighbors[i] && neighbors[i].length > 8) {
	      ifAdj[ifs[i]] = neighbors[i];
	  }
      }
      return ifAdj;
  }

  var weights;
  var hostIfs;
  var hosts;
  var hostLocations;

  var adj = {};
  var agents = [];

  if(source && destination) {
      checkpoint("src_and_dst");
      hosts = [source, destination];
      hostLocations = n.locationMap(hosts);
      var scols = "sourceaddress";
      var scold = "destinationaddress";
      var stype = n.addressType(source);
      switch(stype) {
      case 'uuid': scols = 'uuidsource'; scold = 'uuiddestination'; break;
      case 'ipv4': scols = 'ipsource';  scold = 'ipdestination';  break;
      case 'ipv6': scols = 'ip6source'; scold = 'ip6destination'; break;
      case 'mac' : scols = 'macsource'; scold = 'macdestination'; break;
      }

      var dcols = "sourceaddress";
      var dcold = "destinationaddress";
      var dtype = n.addressType(destination);
      switch(dtype) {
      case 'uuid': dcols = 'uuidsource'; dcold = 'uuiddestination'; break;
      case 'ipv4': dcols = 'ipsource';  dcold = 'ipdestination';  break;
      case 'ipv6': dcols = 'ip6source'; dcold = 'ip6destination'; break;
      case 'mac' : dcols = 'macsource'; dcold = 'macdestination'; break;
      }

      var path_filt = "(" + scols + "=" + source + " & " + dcold + "=" + destination + ") | (" + dcols + "=" + destination + " & " + scold + "=" + source + ")";
      var where = trafficFilter();
      if(where) where = "(" + where + ") & (" + path_filt + ")";
      else where = path_filt;
	 
      var ifs = {};
      var myWeights = {};
      var maxval = 0;
      var q = Query.topN(
			 "rtfanout",
			 [ "inputinterface,llbytes","outputinterface,llbytes" ],
			 [ path_filt, where ],
			 webapp.getValue("interval","last15minutes"),
			 "llbytes",
			 100);
      q.multiquery = true;
      q.nullsinkeys = true;
      checkpoint("src_and_dst query...");
      q.run(
	  function(row) { 
	      var ifx = row[0];
	      if(ifx) {
		  ifs[ifx] = true;
		  accumulateMax(myWeights, ifx, row[1]);
	      }
	  }
      ); 
      checkpoint("src_and_dst filter");

      // use full adjacency to help find connected interfaces
      // expensive operation
      n.path = null;
      adj = interfaceAdjacency();

      var newAgents = {};
      agents = [];
      newAdj = {};
      hostIfs = {};

      for(var ifx in ifs) {
	  n.path = ifx;
	  var agent = ifx_to_IP(ifx); // n.agentIP();
	  if(agent && !newAgents[agent]) {
	      newAgents[agent] = true;
	      agents.push(agent);
	  }
	  // use the full adjacency to expand to interfaces
	  // that we know are at the other end of the link...
	  ifx2 = adj[ifx];
	  if(ifx2) {
	      newAdj[ifx] = ifx2;
	      newAdj[ifx2] = ifx;
	      n.path = ifx2;
	      agent = ifx_to_IP(ifx2); // n.agentIP();
	      if(agent && !newAgents[agent]) {
		  newAgents[agent] = true;
		  agents.push(agent);
	      }
	  }
	  else {
	      newAdj[ifx] = ifx;
	      hostIfs[ifx] = true;
	  }
      }
      for each (var ifx in hostLocations) {
	  if(!newAdj[ifx]) newAdj[ifx] = ifx;
      }

      adj = newAdj;

      if("traffic" == webapp.getValue("show","links")) {
	  weights = myWeights;
	  normalizeWeights(weights,maxval);
      }
      checkpoint("src_and_dst done");
  }

  // calculate link weights first since we 
  // might want to suppress unconnected nodes

  if(!weights) weights = calculateLinkWeights(adj);

  var statusFlag = webapp.getValue("show","status") == "status";

  var status = {};
  var statuslist = n.statusMap(agents,["status"]);
  for (var i = 0; i < agents.length; i++) {
     if(statuslist[i] && statuslist[i].length > 0) {
       status[agents[i]] = statuslist[i][0];
     }
  }

  // add nodes
  checkpoint("add nodes...");
  for each (var ip in agents) {
    n.path = ip;
    addAgent(n.agent(),n.agentIP(),status[ip]);
  }

  // add on hosts if needed
  if(hosts) {
     checkpoint("add hosts...");
     var names = n.dnsMap(hosts);
     status = {};
     if(statusFlag) {
       statuslist = n.statusMap(hostLocations,["link"]);
       for(var i = 0; i < hostLocations.length; i++) {
         if(statuslist[i] && statuslist[i].length > 0) {
           status[hostLocations[i]] = statuslist[i][0];
         }
       }
     }
     for (var i = 0; i < hosts.length; i++) {
        var host = hosts[i];
        var name = names[i] && names[i].length == 1 ? names[i][0] : host;
        addHost(name,host);

        var location = hostLocations[i];
        if(location) {
          n.path = location;
          var agent = n.agentIP();
          if(agent && nodes[agent]) {
             addHostEdge(location + "," + host, agent, host, weights[location],status[location]);
          }
        }
     }
  }
	
  // add edges
  checkpoint("edge status...");
  status = {};
  if(statusFlag) {
    var portlist = [];
    for(var if1 in adj) portlist.push(if1);
    checkpoint("statusMap...");
    statuslist = n.statusMap(portlist,["link"]);
    checkpoint("statusMap done");
    for(var i = 0; i < portlist.length; i++) {
      if(statuslist[i] && statuslist[i].length > 0) {
        status[portlist[i]] = statuslist[i][0];
      }
    }
  }
  checkpoint("add edges...");
  for (var if1 in adj) {
    var if2 = adj[if1];
    if(if2 != if1) {
      var edkey = if2 < if1 ? if2+","+if1 : if1+","+if2;
      if(!edges[edkey]) {
          n.path = if1;
          var node1 = ifx_to_IP(if1); // n.agentIP();
          n.path = if2;
          var node2 = ifx_to_IP(if2); // n.agentIP();
	  if(node1 && node2) {
              var mstatus = null;
              if(statusFlag) {
		  var status1 = status[if1];
		  var status2 = status[if2];
		  mstatus = n.maxStatus(status1,status2);
              }
              var weight = Math.max(
		  weights[if1] ? weights[if1] : 0,
		  weights[if2] ? weights[if2] : 0
              );
              addEdge(edkey,node1,node2,weight,mstatus);
	  }  
      }
    }
  }
  checkpoint("add edges done");
  return graph;
}

function header() {
    var show = webapp.queryParameter("show",webapp.getValue("show"));
    var source = webapp.queryParameter("source",webapp.getValue("source"));
    var destination = webapp.queryParameter("destination",webapp.getValue("destination"));
    var filter = [ 
	webapp.sourceAndDestination(source, destination),
    ];
    var subfilter = [
        webapp.spinBox("show","t.show","status",["status","ifSpeed","bps","traffic"],[webapp.localize("map.show.status"),webapp.localize("map.show.ifSpeed"),webapp.localize("map.show.bps"),webapp.localize("map.show.traffic")])
    ];
    if("traffic" == show) {
	// subfilter.push(webapp.spinBox("host","t.host",null,"host"));
	// subfilter.push(webapp.spinBox("protocol","t.protocol",null,"protocol"));
    }
    if("bps" == show 
       || "traffic" == show
       || (webapp.getValue("source") && webapp.getValue("destination"))) {
	subfilter.push(webapp.spinBox("interval","t.interval","last15minutes","interval"));
    }
    
    if("traffic" == show) {
	webapp.filterBar(filter,subfilter, "");
    }
    else webapp.filterBar(filter,subfilter);
}

function data(evtName, evtId, evtValue) {

    var resp = {};

    switch(evtName) {
    case "nodeClick":
	switch(evtId) {
	case "agent":
	    n.path = evtValue;
	    resp.action = "dialog";
	    resp.title = n.agent();
	    resp.values = [];
	    resp.values.push({label:webapp.localize("map.enterprise"),value:n.enterpriseName()});
	    resp.values.push({label:webapp.localize("config.zone"),value:n.zone(),evtId:'zone',evtValue:evtValue});
	    resp.values.push({label:webapp.localize("config.group"),value:n.group(),evtId:'group',evtValue:evtValue});
	    resp.values.push({label:webapp.localize("map.search"),value:evtValue,evtId:'searchAgent'});
	    break;
         case "host":
           resp.action = "dialog";
	   var parts = evtValue.split(",");
	   var addr = parts[0];
	   var name = parts[1];
           resp.title = name;
           resp.values = [];
	   resp.values.push({label:webapp.localize("map.search"), value:addr, evtId:'searchHost', evtValue: addr});
           break;
	}
	break;
    case "edgeClick":
        switch(evtId) {
        case "edge":
            var parts = evtValue.split(',');
            resp.action = "dialog";
            resp.title = webapp.localize("map.link");
            resp.values = [];
            n.path = parts[0];
            resp.values.push({label:"Agent 1", value: n.agent(),evtId:'searchAgent',evtValue:n.agentIP()});
            resp.values.push({label:"Port 1", value: n.displayName(),evtId:'searchAgent',evtValue:n.path});
            n.path = parts[1];
            resp.values.push({label:"Agent 2", value: n.agent(),evtId:'searchAgent',evtValue:n.agentIP()});
            resp.values.push({label:"Port 2", value: n.displayName(),evtId:'searchAgent',evtValue:n.path});
            break;
        }	
	break;
    case "dialogClick":
        switch(evtId) {
        case "zone":
            resp.action = "loadgraph";
            resp.refreshLayout = true;
            n.path = evtValue;
            webapp.setValue("path",n.zonePath());
            break;
        case "group":
            resp.action = "loadgraph";
            resp.refreshLayout = true;
            n.path = evtValue;
            webapp.setValue("path",n.groupPath());
            break;
        case "searchAgent":
	    resp.action = "loadpage";
	    resp.url = webapp.encodeRedirectURL('Go?action=agent&agent=' + evtValue);
            break;
        case "searchHost":
	    resp.action = "loadpage";
	    resp.url = webapp.encodeRedirectURL('Go?action=host&host=' + evtValue);
            break;
        }
        break;
    case "layout":
	// the layout option was changed - capture the new value so it can be the default next time
	webapp.setValue("layout", evtValue);
	break;
    default:
	resp.action = "loadgraph";
	break;
  }
    
  if(resp.action == "loadgraph") {
    var source = webapp.getValue("source");
    var destination = webapp.getValue("destination");
    resp.graph = createMap(source,destination);
  }

  checkpoint("end");
  resp.performance=performance;

  // write out the JSON result
  println(JSON.stringify(resp, null, 2));
}

var source = webapp.queryParameter("source");
if(source) {
  webapp.setValue("source",source);
}
var destination = webapp.queryParameter("destination");
if(destination) {
  webapp.setValue("destination",destination); 
}

webapp.graph(
    header,
    data, 
    {
	layout: webapp.getValue("layout") || preference("map.path.layout") || "organic",
        refresh: true,
        height: webapp.getValue("com.inmon.servlets.map.height"),
	panLabel: webapp.localize("map.pan"),
	upLabel: webapp.localize("map.pan.up"),
	downLabel: webapp.localize("map.pan.down"),
	leftLabel: webapp.localize("map.pan.left"),
	rightLabel: webapp.localize("map.pan.right"),
	zoomLabel: webapp.localize("map.zoom"),
	inLabel: webapp.localize("map.zoom.in"),
	outLabel: webapp.localize("map.zoom.out"),
	layoutLabel: webapp.localize("map.layout"),
	organicLabel: webapp.localize("map.layout.organic"),
	orthogonalLabel: webapp.localize("map.layout.orthogonal"),
	treeLabel: webapp.localize("map.layout.tree"),
	circularLabel: webapp.localize("map.layout.circular"),
	hierarchicLabel: webapp.localize("map.layout.hierarchic"),
	resetZoomLabel: webapp.localize("map.zoom.reset")
    });

