// path-filter: nav.monitor

var app = WebApp.current();
var chartSpec = app.getValue("com.inmon.servlets.monitor.jsonFlowSpec",null);
var spec = chartSpec ? JSON.parse(chartSpec) : null;

var otherProtocolStr = app.localize("monitor.circles.otherprotocol");
var otherClusterStr = app.localize("monitor.circles.othercluster");

String.prototype.startsWith = function(str) {
  return (this.match("^"+str)==str);
}

function simplifyFilter(filter) {
  var query = new Query();
  query.view = spec.view;
  query.where = filter;
  return query.simplifiedFilter();
}

var server_cluster,client_cluster;
switch(spec.cluster) {
  case 'zone':
    server_cluster = "serverzone";
    client_cluster = "clientzone";
    break;
  case 'group':
    server_cluster = "servergroup";
    client_cluster = "clientgroup";
    break;
  case 'subnet':
    server_cluster = "serversubnet";
    client_cluster = "clientsubnet";
    break;
  case 'agent':
    server_cluster = "serveraddress";
    client_cluster = "clientaddress";
    break;
  default:
    server_cluster = "serverzone";
    client_cluster = "clientzone";
}


function data() {
  if(!spec) return; 

  var network = Network.current();

  // add name columns to the query (if needed)

  var refresh = "true" == app.queryParameter("refresh","false");

  var query = Query.topN(
    spec.view,
    server_cluster + ",serveraddress,last(servername)," + client_cluster + ",clientaddress,last(clientname),serverport," + spec.resultSort,
    spec.resultFilter,
    spec.interval,
    spec.resultSort,
    spec.nFlows);

  var table = query.run();
  if(!table) return;

  if(spec.cluster == "agent") {
     var addrToAgent = {};
     for(var r = 0; r < table.nrows; r++) {
        var srcaddr = table.cell(r,0);
        if(srcaddr) addrToAgent[srcaddr] = otherClusterStr;
        var dstaddr = table.cell(r,3);
        if(dstaddr) addrToAgent[dstaddr] = otherClusterStr;
     }
     var alist = [];
     for(var addr in addrToAgent) alist.push(addr);
     var locations = network.locationMap(alist);
     for(var i = 0; i < alist.length; i++) {
        var loc = locations[i];
        if(loc) {
           network.path = loc;
           var agt = network.agentIP();
           if(agt) addrToAgent[alist[i]] = agt;
        }
     }
     for(var r = 0; r < table.nrows; r++) {
        table.cdata[0][r] = addrToAgent[table.cell(r,0)];
        table.cdata[3][r] = addrToAgent[table.cell(r,3)];
     }
  }

  var protocols = {};
  for(var r = 0; r < table.nrows; r++) {
     var port = table.cell(r,6);
     var val = table.cell(r,7);
     if(port && val) {
       if(protocols[port]) protocols[port]++;
       else protocols[port] = 1;
     }
  }

  var plist = [];
  for(var port in protocols) plist.push(port);
  plist.sort(function(a,b) { return (protocols[b] - protocols[a]) });
  var topProtocols = {};
  for(var i = 0; i < Math.min(plist.length,spec.nProtocols); i++) {
    topProtocols[plist[i]] = "" + i;
  }

  var addressMap = {};
  var addrs = {};
  var addressToGroup = {};
  var groups = {};
  var flows = {};
  for(var r = 0; r < table.nrows; r++) {
     var sourcegroup = table.cell(r,0);
     var sourceaddress = table.cell(r,1);
     var sourcename = table.cell(r,2);
     var destinationgroup = table.cell(r,3);
     var destinationaddress = table.cell(r,4);
     var destinationname = table.cell(r,5);
     var port =  table.cell(r,6);
     var val = table.cell(r,7);
     if(sourcegroup 
        && sourceaddress
        && destinationgroup
        && destinationaddress
        && val) {
        addressToGroup[sourceaddress] = sourcegroup;
        groups[sourcegroup] = -1;
        addressToGroup[destinationaddress] = destinationgroup;
        groups[destinationgroup] = -1;
        var flowkey;
        if(sourceaddress < destinationaddress) flowkey = sourceaddress + "," + destinationaddress;
        else flowkey = destinationaddress + "," + sourceaddress;
        var protocol = topProtocols[port];
        if(!protocol) protocol = spec.nProtocols;
        flowkey += "," + protocol;
        if(flows[flowkey]) flows[flowkey] += val;
        else flows[flowkey] = val;
        if(sourcename) addressMap[sourceaddress] = network.truncateDNS(sourcename);
	else {
           var atype = network.addressType(sourceaddress);
           if('ipv4' == atype || 'ipv6' == atype) addrs[sourceaddress] = sourceaddress;
        }
        if(destinationname) addressMap[destinationaddress] = network.truncateDNS(destinationname);
        else {
           var atype = network.addressType(destinationaddress);
           if('ipv4' == atype || 'ipv6' == atype) addrs[destinationaddress] = destinationaddress;
        }
     }
  }

  var addrlist = [];
  for (var addr in addrs) addrlist.push(addr);
  if(addrlist.length > 0) {
    var names = network.dnsMap(addrlist);
    for(var i = 0; i < addrlist.length; i++) {
       var namelist = names[i];
       if(namelist && namelist.length == 1) {
         if(!addressMap) addressMap = {};
         addressMap[addrlist[i]] = namelist[0];
       }
    }
  }

  var grouplist = [];
  for(var group in groups) {
    var idx = groups[group];
    if(idx == -1) {
      idx = grouplist.length;
      grouplist[idx] = group;
      groups[group] = idx;
    } 
  }

  var result = {};

  result.protocols = [];
  var otherlist = "OTHER-";
  for(var i = 0; i < Math.min(plist.length,spec.nProtocols); i++) {
    result.protocols.push([plist[i],network.protocolName(plist[i],true)]);
    if(i > 0) otherlist += ",";
    otherlist += plist[i];
  }
  if(plist.length > spec.nProtocols) result.protocols.push([otherlist,otherProtocolStr]);  

  result.groups = grouplist;
  result.nodeNames = addressMap;

  result.flows = [];
  for(var flowkey in flows) {
    var parts = flowkey.split(',');
    result.flows.push([
		       groups[addressToGroup[parts[0]]],
		       parts[0],
		       groups[addressToGroup[parts[1]]],
		       parts[1],
		       parts[2],
		       flows[flowkey]]);
  }
  println(JSON.stringify(result));
}

function updateHistory(ctype,cvalue) {
   if('address' == ctype) app.addressToHistory(cvalue);
   else if('port' == ctype) app.protocolToHistory(cvalue);
   else if('agent' == ctype) app.protocolToHistory(cvalue);
}

function action(evtName,evtId,evtValue) {
  switch(evtName) {
    case "nodeClick":
       var response = {};
       response.action = "dialog";
       response.selectFlag = true;
       response.excludeFlag = true;
       response.message = '<table class="overview"><tbody>';
       switch(evtId) {
       case "PROTOCOL":
         response.searchFlag = !evtValue.startsWith("OTHER-");
         response.message += '<tr class="even"><th>' + app.localize("c.serverport") + '</th><td>' + (evtValue.startsWith("OTHER-") ? otherProtocolStr : evtValue) + '</td></tr>';
         break;
       case "NODE":
         response.searchFlag = true;
         var parts = evtValue.split(",");
         response.message += '<tr class="even"><th>' + app.localize("monitor.circles.nodeaddress") + '</th><td>' + parts[0] + '</td></tr>';
         if(parts[1] && parts[0] != parts[1]) {
           response.message += '<tr class="odd"><th>' + app.localize("monitor.circles.nodename") + '</th><td>' + parts[1] + '</td></tr>';
         }
         break;
       case "CLUSTER":
         response.searchFlag = spec.cluster == "agent" && evtValue != otherClusterStr;
         response.selectFlag = spec.cluster != "agent";
         response.excludeFlag = spec.cluster != "agent";
         response.message += '<tr class="even"><th>' + app.localize("monitor.cluster." + spec.cluster) + '</th><td>' + evtValue + '</td></tr>';
         break;
       case "LINK":
         response.searchFlag = false;
         var parts = evtValue.split(",");
         var startaddr = parts[0];
         var startname = parts[1];
         var endaddr = parts[2];
         var endname = parts[3];
         response.message += '<tr class="even"><th>' + app.localize("monitor.circles.node1address") + '</th><td>' + startaddr + '</td></tr>';
         response.message += '<tr class="odd"><th>' + app.localize("monitor.circles.node1name") + '</th><td>' + (startname && startname != startaddr ? startname : "") + '</td></tr>';
         response.message += '<tr class="even"><th>' + app.localize("monitor.circles.node2address") + '</th><td>' + endaddr + '</td></tr>';
         response.message += '<tr class="odd"><th>' + app.localize("monitor.circles.node2name") + '</th><td>' + (endname && endname != endaddr ? endname : "") + '</td></tr>';
         break;
       }
       response.message += '</tbody></table>';
       println(JSON.stringify(response,null,2));
       break;
    case "select":
       var response = {};
       response.action = "loadpage";
       var filter = app.getValue("com.inmon.servlets.monitor.filter." + spec.view,"");
       switch(evtId) {
       case "PROTOCOL":
         if(evtValue.startsWith("OTHER-")) {
           var protocols = evtValue.substring("OTHER-".length);
           if(filter) filter = "(" + filter + ") & serverport != " + protocols;
           else filter = "serverport != " + protocols;
         }
         else {
           updateHistory("port",evtValue);
           if(filter) filter = "(" + filter + ") & serverport = " + evtValue;
           else filter = "serverport = " + evtValue;
         }
         break;
       case "NODE":
         var parts = evtValue.split(",");
         var addr = parts[0];
         updateHistory("address",addr);
         if(filter) filter = "(" + filter + ") & (serveraddress = " + addr + " | clientaddress = " + addr + ")";
         else filter = "serveraddress = " + addr + " | clientaddress = " + addr;
         break;
       case "CLUSTER":
         if(filter) filter = "(" + filter + ") & (" + server_cluster + " = '" + evtValue + "' | " + client_cluster + " = '" + evtValue + "')";
         else filter = server_cluster + " = '" + evtValue + "' | " + client_cluster + " = '" + evtValue + "'";
         break;
       case "LINK":
         var parts = evtValue.split(",");
         var startaddr = parts[0];
         var endaddr = parts[2];
         if(filter) filter = "(" + filter + ") & ((serveraddress = " + startaddr + " & clientaddress = " + endaddr + ") | (serveraddress = " + endaddr + " & clientaddress = " + startaddr + "))";
         else filter = "(serveraddress = " + startaddr + " & clientaddress = " + endaddr + ") | (serveraddress = " + endaddr + " & clientaddress = " + startaddr + ")"; 
         break;
       }
       app.setValue("com.inmon.servlets.monitor.filter." + spec.view,simplifyFilter(filter));
       response.url = app.encodeRedirectURL(app.createURL({}));
       println(JSON.stringify(response,null,2));
       break;
    case "exclude":
       var response = {};
       response.action = "loadpage";
       var filter = app.getValue("com.inmon.servlets.monitor.filter." + spec.view,"");
       switch(evtId) {
       case "PROTOCOL":
         if(evtValue.startsWith("OTHER-")) {
           var protocols = evtValue.substring("OTHER-".length);
           if(filter) filter = "(" + filter + ") & serverport = " + protocols;
           else filter = "serverport = " + protocols;        
         }
         else {
           if(filter) filter = "(" + filter + ") & serverport != " + evtValue;
           else filter = "serverport != " + evtValue;
         }
         break;
       case "NODE":
         var parts = evtValue.split(",");
         var addr = parts[0];
         updateHistory("address",addr);
         if(filter) filter = "(" + filter + ") & serveraddress != " + addr + " & clientaddress != " + addr;
         else filter = "serveraddress != " + addr + " & clientaddress != " + addr;
         break;
       case "CLUSTER":
         if(filter) filter = "(" + filter + ") & !(" + server_cluster + " = '" + evtValue + "' | " + client_cluster + " = '" + evtValue + "')";
         else filter = "!(" + server_cluster + " = '" + evtValue + "' | " + client_cluster + " = '" + evtValue + "')";
         break;
       case "LINK":
         var parts = evtValue.split(",");
         var startaddr = parts[0];
         var endaddr = parts[2];
         if(filter) filter = "(" + filter + ") & !((serveraddress = " + startaddr + " & clientaddress = " + endaddr + ") | (serveraddress = " + endaddr + " & clientaddress = " + startaddr + "))";
         else filter = "!(serveraddress = " + startaddr + " & clientaddress = " + endaddr + ") | (serveraddress = " + endaddr + " & clientaddress = " + startaddr + ")";
         break;
       }
       app.setValue("com.inmon.servlets.monitor.filter." + spec.view,simplifyFilter(filter));
       response.url = app.encodeRedirectURL(app.createURL({}));
       println(JSON.stringify(response,null,2));
       break;
    case "search":
       var response = {};
       response.action = "loadpage";
       switch(evtId) {
       case "PROTOCOL": 
          response.url = app.encodeRedirectURL(app.linkForTypeAndValue("port",evtValue));
          break;
       case "NODE":
          response.url = app.encodeRedirectURL(app.linkForTypeAndValue("address",evtValue.split(",")[0]));
          break;
       case "CLUSTER":
          response.url = app.encodeRedirectURL(app.linkForTypeAndValue("agent",evtValue));
          break;
       case "LINK": break;
       }
       println(JSON.stringify(response,null,2));
       break;
    default: data();
  }
}

app.circles(null, action, {
   "height"       : spec.chartHeight,
   "nProtocols"   : spec.nProtocols,
   "colors"       : spec.colors,
   "autolabel"    : spec.autolabel,
   "select"       : app.localize("appletmenu.filter"),
   "exclude"      : app.localize("appletmenu.exclude"),
   "search"       : app.localize("appletmenu.search"),
   "refresh"      : spec.refresh,
   "dataTimeout"  : preference("ajax.timeout") ? preference("ajax.timeout") : 600000
});

